#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from __future__ import with_statement
import sha
from string import letters, digits
from random import choice
from optparse import OptionParser
from cocktail.translations import translations
from cocktail.persistence import (
    datastore,
    mark_all_migrations_as_executed
)
from woost.models import (
    changeset_context,
    Site,
    HierarchicalPublicationScheme,
    DescriptiveIdPublicationScheme,
    Language,
    Action,
    Publishable,
    Document,
    User,
    Role,
    StandardPage,
    URI,
    Controller,
    Template,
    UserView,
    ReadPermission,
    CreatePermission,
    ModifyPermission,
    DeletePermission,
    RenderPermission,
    ReadMemberPermission,
    ModifyMemberPermission,
    CreateTranslationPermission,
    ReadTranslationPermission,
    ModifyTranslationPermission,
    DeleteTranslationPermission,
    ReadHistoryPermission,
    DeleteTrigger,
    CustomTriggerResponse,
    ConfirmDraftPermission,
    EmailTemplate,
    CachingPolicy,
    Extension,
    load_extensions
)

standard_template_identifiers = {
    "cocktail": "woost.views.StandardView",
    "genshi": "woost.views.StandardView"
}

def init_site(
    admin_email = "admin@localhost",
    admin_password = "",
    languages = ("en",),
    uri = "/",
    template_engine = "cocktail",
    extensions = ()):

    datastore.root.clear()
    datastore.commit()
    datastore.close()

    def set_translations(item, member, key, **kwargs):
        for language in languages:
            value = translations(
                "woost.models.initialization " + key,
                language,
                **kwargs
            )
            if value:
                item.set(member, value, language)

    # Create standard actions
    create = Action()
    create.identifier = "create"
    set_translations(create, "title", "Create action title")
    create.insert()

    read = Action()
    read.identifier = "read"
    set_translations(read, "title", "Read action title")
    read.insert()

    modify = Action()
    modify.identifier = "modify"
    set_translations(modify, "title", "Modify action title")
    modify.insert()

    delete = Action()
    delete.identifier = "delete"
    set_translations(delete, "title", "Delete action title")
    delete.insert()

    confirm_draft = Action()
    confirm_draft.identifier = "confirm_draft"
    set_translations(confirm_draft, "title", "Confirm draft title")
    confirm_draft.insert()

    with changeset_context() as changeset:

        # Create the site
        site = Site()
        site.qname = "woost.main_site"
        site.insert()
        site.secret_key = random_string(10)

        # Create the administrator user
        admin = User()
        admin.author = admin
        admin.qname = "woost.administrator"
        admin.owner = admin
        admin.critical = True
        admin.email = admin_email
        admin.password = admin_password
        admin.insert()

        changeset.author = admin
        site.author = site.owner = admin
        site.default_language = languages[0]

        if site.default_language in Site.backoffice_language.enumeration:
            site.backoffice_language = site.default_language
            admin.prefered_language = site.default_language

        # Create the anonymous user and role
        anonymous_role = Role()
        anonymous_role.implicit = True
        anonymous_role.critical = True
        anonymous_role.qname = "woost.anonymous"
        set_translations(anonymous_role, "title", "Anonymous role title")
        anonymous_role.insert()

        anonymous_user = User()
        anonymous_user.email = "anonymous@localhost"
        anonymous_user.qname = "woost.anonymous_user"
        anonymous_user.anonymous = True
        anonymous_user.critical = True
        anonymous_user.roles.append(anonymous_role)
        anonymous_user.insert()

        # Create languages
        for code in languages:
            language = Language(iso_code = code)
            language.iso_code = code
            language.insert()

        # Create the administrators role
        administrators = Role()
        administrators.qname = "woost.administrators"
        administrators.critical = True
        set_translations(administrators, "title", "Administrators role title")
        administrators.users.append(admin)
        everything = lambda: {"type": "woost.models.item.Item"}
        administrators.permissions = [
            # Administrators have full control
            ReadPermission(matching_items = everything()),
            CreatePermission(matching_items = everything()),
            ModifyPermission(matching_items = everything()),
            DeletePermission(matching_items = everything()),
            ConfirmDraftPermission(matching_items = everything()),
            ReadMemberPermission(),
            ModifyMemberPermission()
        ]
        administrators.insert()

        # Create the 'everybody' role
        everybody_role = Role()
        everybody_role.implicit = True
        everybody_role.critical = True
        everybody_role.qname = "woost.everybody"
        set_translations(everybody_role, "title", "Everybody role title")
        owned_items = lambda: {
            "type": "woost.models.item.Item",
            "filter": "owned-items"
        }
        everybody_role.permissions = [

            # Everybody can render any item
            RenderPermission(
                matching_items = {
                    "type": "woost.models.item.Item"
                }
            ),

            # Everybody can read published items
            ReadPermission(
                matching_items = {
                    "type": "woost.models.publishable.Publishable",
                    "filter": "published"
                }
            ),

            # Content owners have full control
            ModifyPermission(matching_items = owned_items()),
            DeletePermission(matching_items = owned_items()),
            ConfirmDraftPermission(matching_items = owned_items()),

            # All members allowed, except for 'local_path', 'controller' and 'qname'
            ReadMemberPermission(
                matching_members = [
                    "woost.models.item.Item.qname",
                    "woost.models.publishable.Publishable.controller",
                    "woost.models.file.File.local_path"
                ],
                authorized = False
            ),

            # Only administrators can see the CachePolicy model
            ReadPermission(
                matching_items = {
                    "type": "woost.models.caching.CachingPolicy"
                },
                authorized = False
            ),

            ReadMemberPermission(),

            # Only administrators can modify 'owner', 'robots_should_index',
            # 'robots_should_follow' and 'requires_https' members
            ModifyMemberPermission(
                matching_members = ["woost.models.item.Item.owner"],
                authorized = False
            ),
            ModifyMemberPermission(
                matching_members = [
                    "woost.models.publishable.Publishable.requires_https"
                ],
                authorized = False
            ),
            ModifyMemberPermission(
                matching_members = [
                    "woost.models.document.Document.robots_should_index",
                    "woost.models.document.Document.robots_should_follow"
                ],
                authorized = False
            ),
            ModifyMemberPermission(),

            # All languages allowed
            CreateTranslationPermission(),
            ReadTranslationPermission(),
            ModifyTranslationPermission(),
            DeleteTranslationPermission()
        ]
        everybody_role.insert()

        # Create the 'authenticated' role
        authenticated_role = Role()
        authenticated_role.implicit = True
        authenticated_role.critical = True
        authenticated_role.qname = "woost.authenticated"
        set_translations(authenticated_role, "title",
            "Authenticated role title")
        authenticated_role.permissions = [
            ReadHistoryPermission()
        ]
        authenticated_role.insert()

        # Create publication schemes
        for pub_scheme in (
            HierarchicalPublicationScheme(),
            DescriptiveIdPublicationScheme()
        ):
            site.publication_schemes.append(pub_scheme)
            pub_scheme.insert()

        # Create a trigger to purge deleted files from the filesystem
        delete_files_trigger = DeleteTrigger(
            execution_point = "after",
            batch_execution = True,
            matching_items = {"type": "woost.models.file.File"},
            responses = [
                CustomTriggerResponse(
                    code = u"from os import remove\n"
                           u"for item in items:\n"
                           u"    remove(item.file_path)"
                )
            ]
        )
        set_translations(delete_files_trigger, "title",
            "delete_files_trigger"
        )
        site.triggers.append(delete_files_trigger)
        delete_files_trigger.insert()

        # Create standard controllers
        for controller_name in (
            "Document",
            "File",
            "URI",
            "Styles",
            "Feed",
            "WebServices",
            "BackOffice",
            "FirstChildRedirection",
            "Login",
            "PasswordChange",
            "PasswordChangeConfirmation"
        ):
            controller = Controller()
            controller.qname = "woost.%s_controller" % controller_name.lower()
            set_translations(
                controller,
                "title",
                controller_name + " controller title"
            )
            controller.python_name = \
                "woost.controllers.%scontroller.%sController" % (
                    controller_name.lower(),
                    controller_name
                )
            controller.insert()

        # The backoffice controller is placed at an irregular location
        back_office_controller = \
            Controller.get_instance(qname = "woost.backoffice_controller")
        back_office_controller.python_name = \
            "woost.controllers.backoffice.backofficecontroller." \
            "BackOfficeController"

        # Prevent anonymous access to the backoffice controller
        ReadPermission(
            role = anonymous_role,
            matching_items = {
                "type": "woost.models.publishable.Publishable",
                "filter": "member-controller",
                "filter_operator0": "eq",
                "filter_value0": str(back_office_controller.id)
            },
            authorized = False
        ).insert()

        # Create standard templates
        std_template = Template()
        std_template.identifier = standard_template_identifiers.get(
            template_engine, "StandardView"
        )
        std_template.engine = template_engine
        std_template.qname = "woost.standard_template"
        set_translations(std_template, "title", "Standard template title")
        std_template.insert()

        # Create Login Form view
        login_form_template = Template()
        login_form_template.identifier = u"woost.views.LoginFormView"
        login_form_template.engine = u"cocktail"
        set_translations(
            login_form_template,
            "title", "Login Form template title"
        )
        login_form_template.insert()

        # Create Password Change Request view
        password_change_request_view = Template()
        password_change_request_view.identifier = \
            u"woost.views.PasswordChangeRequestView"
        password_change_request_view.engine = u"cocktail"
        set_translations(
            password_change_request_view,
            "title", "Password Change Request template title"
        )
        password_change_request_view.insert()

        # Create Password Change Email template
        password_change_confirmation_email_template = EmailTemplate()
        password_change_confirmation_email_template.template_engine = u"mako"
        password_change_confirmation_email_template.qname = \
            u"woost.views.password_change_confirmation_email_template"
        set_translations(
            password_change_confirmation_email_template,
            "title", "Password Change Confirmation Email template title"
        )
        set_translations(
            password_change_confirmation_email_template,
            "subject", "Password Change Confirmation Email subject"
        )
        set_translations(
            password_change_confirmation_email_template,
            "body", "Password Change Confirmation Email body"
        )
        password_change_confirmation_email_template.set(
            "sender",
            u'u"'+admin.email+'"'
        )
        password_change_confirmation_email_template.set(
            "receivers",
            u"[user.email]"
        )
        password_change_confirmation_email_template.insert()

        # Create Password Change Confirmation view
        password_change_confirmation_view = Template()
        password_change_confirmation_view.identifier = \
            u"woost.views.PasswordChangeConfirmationView"
        password_change_confirmation_view.engine = u"cocktail"
        set_translations(
            password_change_confirmation_view,
            "title", "Password Change Confirmation view title"
        )
        password_change_confirmation_view.insert()

        # Create standard resources
        site_stylesheet = URI()
        site_stylesheet.uri = uri + "resources/styles/site.css"
        site_stylesheet.mime_type = "text/css"
        site_stylesheet.qname = "woost.site_stylesheet"
        set_translations(site_stylesheet, "title", "Site style sheet title")
        site_stylesheet.insert()

        # Create the temporary home page
        site.home = StandardPage()
        site.home.template = std_template
        site.home.qname = "woost.home"
        set_translations(site.home, "title", "Home page title")
        set_translations(site.home, "inner_title", "Home page inner title")
        set_translations(
            site.home, "body", "Home page body",
            uri = uri + "cms"
        )
        site.home.branch_resources.append(site_stylesheet)
        site.home.insert()

        # Create the back office interface
        back_office = Document()
        back_office.controller = back_office_controller
        back_office.critical = True
        back_office.qname = "woost.backoffice"
        back_office.per_language_publication = False
        back_office.parent = site.home
        back_office.hidden = True
        back_office.path = u"cms"
        back_office.inherit_resources = False
        set_translations(back_office, "title", "Back office title")
        back_office.insert()

        # Create the user styles dynamic style sheet
        user_styles = Document()
        user_styles.critical = True
        user_styles.qname = "woost.user_styles"
        user_styles.per_language_publication = False
        user_styles.parent = site.home
        user_styles.controller = \
            Controller.get_instance(qname = "woost.styles_controller")
        user_styles.hidden = True
        user_styles.path = u"user_styles"
        user_styles.mime_type = "text/css"
        user_styles.caching_policies.append(
            CachingPolicy(
                server_side_cache = True,
                last_update_expression =
                    "from woost.models import Style\n"
                    "last_update = [publishable, latest(Style)]\n"
            )
        )
        set_translations(user_styles, "title", "User styles title")
        user_styles.insert()

        # Create the web services page
        webservices = Document()
        webservices.critical = True
        webservices.qname = "woost.webservices"
        webservices.parent = site.home
        webservices.controller = \
            Controller.get_instance(qname = "woost.webservices_controller")
        webservices.hidden = True
        webservices.path = u"services"
        webservices.mime_type = "application/json"
        set_translations(webservices, "title", "Web services title")
        webservices.insert()

        # Create the 'content not found' page
        site.not_found_error_page = StandardPage()
        site.not_found_error_page.parent = site.home
        site.not_found_error_page.hidden = True
        site.not_found_error_page.template = std_template
        site.not_found_error_page.qname = "woost.not_found_error_page"
        set_translations(site.not_found_error_page, "title",
            "Not found error page title")
        set_translations(site.not_found_error_page, "body",
            "Not found error page body")
        site.not_found_error_page.insert()

        # Create forbidden error page
        site.forbidden_error_page = StandardPage()
        site.forbidden_error_page.parent = site.home
        site.forbidden_error_page.hidden = True
        site.forbidden_error_page.template = std_template
        site.forbidden_error_page.qname = "woost.forbidden_error_page"
        set_translations(site.forbidden_error_page, "title",
            "Forbidden error page title")
        set_translations(site.forbidden_error_page, "body",
            "Forbidden error page body")
        site.forbidden_error_page.insert()

        # Create the password change request page
        site.password_change_page = StandardPage()
        site.password_change_page.parent = site.home
        site.password_change_page.hidden = True
        site.password_change_page.template = password_change_request_view
        site.password_change_page.controller = \
            Controller.get_instance(qname='woost.passwordchange_controller')
        site.password_change_page.qname = "woost.password_change_page"
        site.password_change_page.per_language_publication = False
        set_translations(site.password_change_page, "title",
            "Password Change page title")
        set_translations(site.password_change_page, "body",
            "Password Change page body")
        site.password_change_page.insert()

        # Create the password change confirmation page
        site.password_change_confirmation_page = StandardPage()
        site.password_change_confirmation_page.parent = site.home
        site.password_change_confirmation_page.hidden = True
        site.password_change_confirmation_page.per_language_publication = False
        site.password_change_confirmation_page.template = \
            password_change_confirmation_view
        site.password_change_confirmation_page.controller = \
            Controller.get_instance(
                qname='woost.passwordchangeconfirmation_controller'
            )
        site.password_change_confirmation_page.qname = \
            "woost.password_change_confirmation_page"
        set_translations(site.password_change_confirmation_page, "title",
            "Password Change Confirmation page title")
        set_translations(site.password_change_confirmation_page, "body",
            "Password Change Confirmation page body")
        site.password_change_confirmation_page.insert()

        # Create the login page
        site.login_page = StandardPage()
        site.login_page.parent = site.home
        site.login_page.hidden = True
        site.login_page.template = login_form_template
        site.login_page.controller = \
            Controller.get_instance(qname='woost.login_controller')
        site.login_page.qname = "woost.login_page"
        set_translations(site.login_page, "title", "Login page title")
        site.login_page.insert()

        # Create site-wide user views
        own_items_view = UserView()
        own_items_view.roles.append(everybody_role)
        own_items_view.parameters = {
            "type": "woost.models.item.Item",
            "content_view": "flat",
            "filter": "owned-items",
            "order": "-last_update_time",
            "members": None
        }
        set_translations(
            own_items_view,
            "title",
            "Own items user view"
        )
        own_items_view.insert()

        page_tree_view = UserView()
        page_tree_view.roles.append(everybody_role)
        page_tree_view.parameters = {
            "type": "woost.models.publishable.Publishable",
            "content_view": "tree",
            "filter": None,
            "members": None
        }
        set_translations(
            page_tree_view,
            "title",
            "Page tree user view"
        )
        page_tree_view.insert()

        file_gallery_view = UserView()
        file_gallery_view.roles.append(everybody_role)
        file_gallery_view.parameters = {
            "type": "woost.models.file.File",
            "content_view": "thumbnails",
            "filter": None,
            "order": None,
            "members": None
        }
        set_translations(
            file_gallery_view,
            "title",
            "File gallery user view"
        )
        file_gallery_view.insert()

    # Enable the selected extensions
    if extensions:
        load_extensions()
        for extension in Extension.select():
            ext_name = extension.__class__.__name__[:-len("Extension")].lower()
            if ext_name in extensions:
                extension.enabled = True

    mark_all_migrations_as_executed()
    datastore.commit()

def random_string(length, source = letters + digits + "!?.-$#&@*"):
    return "".join(choice(source) for i in range(length))

def main():

    parser = OptionParser()
    parser.add_option("-u", "--user", help = "Administrator email")
    parser.add_option("-p", "--password", help = "Administrator password")
    parser.add_option("-l", "--languages",
        help = "Comma separated list of languages")
    parser.add_option("-t", "--template-engine",
        default = "cocktail",
        help = "The buffet templating engine to use by default")
    parser.add_option("-e", "--extensions",
        default = "",
        help = "The list of extensions to enable")

    options, args = parser.parse_args()

    admin_email = options.user
    admin_password = options.password

    if admin_email is None:
        admin_email = raw_input("Administrator email: ") or "admin@localhost"

    if admin_password is None:
        admin_password = raw_input("Administrator password: ") \
            or random_string(8)

    languages = options.languages \
        and options.languages.replace(",", " ") \
        or raw_input("Languages: ") or "en"

    init_site(
        admin_email,
        admin_password,
        languages.split(),
        template_engine = options.template_engine,
        extensions = options.extensions.split(",")
    )

    print u"Your site has been successfully created. You can start it by " \
          u"executing the 'run.py' script. An administrator account for the " \
          u"content manager interface has been generated, with the " \
          u"following credentials:\n\n" \
          u"\tEmail:     %s\n" \
          u"\tPassword:  %s\n\n" % (admin_email, admin_password)

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.rendering.contentrenderer import ContentRenderer
from woost.models.rendering.contentrenderersregistry import (
    ContentRenderersRegistry,
    content_renderers,
    icon_renderers
)
from woost.models.rendering.imagefilerenderer import ImageFileRenderer
from woost.models.rendering.imageurirenderer import ImageURIRenderer
from woost.models.rendering.videorenderer import VideoRenderer
from woost.models.rendering.pdfrenderer import PDFRenderer
from woost.models.rendering.htmlrenderer import HTMLRenderer
from woost.models.rendering.iconrenderer import IconRenderer
from woost.models.rendering.factories import image_factory, ImageFactoryMember
from woost.models.rendering.effects import image_effect
from woost.models.rendering.cache import (
    require_rendering,
    clear_image_cache,
    BadRenderingRequest
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from cocktail.pkgutils import resolve
from cocktail.translations import translations
from cocktail.schema import Reference
from cocktail.schema.expressions import (
    PositiveExpression,
    NegativeExpression
)
from cocktail.html import Element, templates
from cocktail.controllers import view_state
from woost.models import Item
from woost.views.contentdisplaymixin import ContentDisplayMixin

Table = templates.get_class("cocktail.html.Table")


class ContentTable(ContentDisplayMixin, Table):

    base_url = None
    inline_draft_copies = True
    entry_selector = "tbody tr.item_row"
    resizable_rows_selector = "tbody tr.item_row"

    def __init__(self, *args, **kwargs):
        Table.__init__(self, *args, **kwargs)
        ContentDisplayMixin.__init__(self)
        self.set_member_sortable("element", False)
        self.set_member_sortable("class", False)

    def _row_added(self, index, item, row):
        if self.inline_draft_copies and isinstance(item, Item):
            for draft in item.drafts:
                draft_row = self.create_row(index, draft)
                self.body.append(draft_row)

    def _fill_head(self):
        Table._fill_head(self)
        if self.head_row.children:
            self.head_row.children[-1].add_class("last")

    def create_row(self, index, item):

        row = Table.create_row(self, index, item)
        row.add_class("item_row")

        if getattr(item, "is_draft", False):
            row.add_class("draft")

        if getattr(item, "draft_source", None):
            row.add_class("nested_draft")

        return row

    def create_element_display(self, item, member):

        display = templates.new("woost.views.ItemLabel")
        display.tag = "label"
        display["for"] = "selection_" + str(item.id)
        display.item = item
        display.referer = self.referer

        if self.inline_draft_copies and item.draft_source:
            display.get_label = lambda: translations(
                "woost.views.ContentTable draft label",
                draft_id = item.draft_id
            )

        return display

    def create_class_display(self, item, member):
        return Element(
                    tag = None,
                    children = [translations(item.__class__.name)])

    def add_header_ui(self, header, column, language):

        # Add visual cues for sorted columns
        sortable = self.get_member_sortable(column)

        if sortable and self.order:
            current_direction = self._sorted_columns.get(
                (column.name, language)
            )

            if current_direction is not None:

                header.add_class("sorted")

                if current_direction is PositiveExpression:
                    header.add_class("ascending")
                    sign = "-"
                elif current_direction is NegativeExpression:
                    header.add_class("descending")

        # Column options
        if sortable or self.get_member_searchable(column):

            menu = header.menu = Element()
            header.append(menu)
            menu.add_class("selector")

            label = menu.label = Element()
            label.add_class("label")
            menu.append(label)
            label.append(header.label)

            if column.translated:
                label.append(header.translation_label)

            options = header.menu.options = self.create_header_options(
                column,
                language
            )
            menu.append(options)

    def create_header_options(self, column, language):

        options = Element()
        options.add_class("selector_content")

        if self.get_member_sortable(column):
            sorting_options = self.create_sorting_options(column, language)
            options.append(sorting_options)

            if column.grouping:
                grouping_options = self.create_grouping_options(
                    column,
                    language
                )
                options.append(grouping_options)

        if self.get_member_searchable(column):
            search_options = self.create_search_options(column, language)
            options.append(search_options)

        return options

    def create_sorting_options(self, column, language):

        if self.order:
            direction = self._sorted_columns.get((column.name, language))
        else:
            direction = None

        order_param = column.name
        if language:
            order_param += "." + language

        options = Element()
        options.add_class("sorting_options")

        title = Element("div")
        title.add_class("options_header")
        title.append(translations("woost.views.ContentTable sorting header"))
        options.append(title)

        asc = options.ascending = Element("a")
        asc.add_class("ascending")
        asc["href"] = "?" + view_state(order = order_param, page = 0)
        asc.append(translations("woost.views.ContentTable sort ascending"))
        options.append(asc)

        if direction is PositiveExpression:
            asc.add_class("selected")

        desc = options.ascending = Element("a")
        desc.add_class("descending")
        desc["href"] = "?" + view_state(order = "-" + order_param, page = 0)
        desc.append(translations("woost.views.ContentTable sort descending"))
        options.append(desc)

        if direction is NegativeExpression:
            desc.add_class("selected")

        return options

    def create_search_options(self, column, language):

        filters = [filter.id for filter in self.filters] \
            if self.filters \
            else []

        filter_id = "member-" + column.name
        filters.append(filter_id)

        options = Element()
        options.add_class("search_options")

        title = Element("div")
        title.add_class("options_header")
        title.append(translations("woost.views.ContentTable search header"))
        options.append(title)

        add_filter = Element("a")
        add_filter.add_class("add_filter")
        add_filter["href"] = "?" + view_state(filter = filters, page = 0)
        add_filter.append(
            translations("woost.views.ContentTable add column filter")
        )
        add_filter.set_client_param("filterId", filter_id)
        options.append(add_filter)

        return options

    def create_grouping_options(self, column, language):

        options = Element()
        options.add_class("grouping_options")

        title = Element("div")
        title.add_class("options_header")
        title.append(translations("woost.views.ContentTable grouping header"))
        options.append(title)

        grouping_class = resolve(column.grouping)
        variants = (None,) + grouping_class.variants

        table = Element("table")
        options.append(table)

        for variant in variants:

            tr = Element("tr")
            table.append(tr)

            td = Element("td")
            td.add_class("variant")
            td.append(grouping_class.translate_grouping_variant(variant))
            tr.append(td)

            for sign in (PositiveExpression, NegativeExpression):
                grouping = grouping_class()
                grouping.member = column
                grouping.language = language
                grouping.sign = sign
                grouping.variant = variant

                td = Element("td")
                td.add_class("sign")
                tr.append(td)

                grouping_link = Element("a")
                grouping_link.add_class("grouping")
                grouping_link["href"] = \
                    "?" + view_state(grouping = grouping.request_value, page = 0)
                grouping_link.append(
                    translations(
                        "cocktail.controllers.grouping.MemberGrouping-"
                        + ("ascending"
                            if sign is PositiveExpression
                            else "descending")
                    )
                )
                td.append(grouping_link)

        return options




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.translations import translations
from cocktail.translations.strings import member_identifier

# Validation errors
#------------------------------------------------------------------------------
translations.define("woost.extensions.recaptcha.schemarecaptchas.ReCaptchaValidationError-instance",
    ca = u"Ho sentim, el CAPTCHA és erroni. Si us plau, intenti-ho de nou.",
    es = u"Lo sentimos, el CAPTCHA es erroneo. Por favor inténtelo de nuevo.",
    en = u"Sorry, you failed the CAPTCHA. Please try again."
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from ZODB.POSException import ConflictError
from cocktail.translations import translations
from cocktail.schema import String, Collection
from cocktail.persistence import datastore, delete_dry_run
from cocktail.controllers import request_property
from woost.models import (
    changeset_context,
    delete_validating,
    get_current_user
)
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.editstack import EditNode
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class DeleteController(BaseBackOfficeController):

    MAX_TRANSACTION_ATTEMPTS = 3

    @request_property
    def selection(self):
        """The selected subset of items that should be deleted.
        @type: L{Item<woost.models.item.Item>} collection
        """
        return self.params.read(
            Collection("selection", items = "woost.models.Item"))

    @request_property
    def delete_dry_run(self):
        visited = set()
        return [delete_dry_run(item, visited) for item in self.selection]

    @request_property
    def action(self):
        """A string identifier indicating the action that has been activated by
        the user.
        @type: str or None
        """
        return self.params.read(String("action"))

    @request_property
    def submitted(self):
        return self.action is not None

    def submit(self):
        # Load the edit stack before deleting any item, to ensure its
        # loaded properly
        stack = self.edit_stack

        if self.action == "confirm_delete":

            user = get_current_user()

            for i in range(self.MAX_TRANSACTION_ATTEMPTS):

                deleted_set = None

                with changeset_context(author = user):
                    for item in self.selection:
                        deleted_set = delete_validating(
                            item,
                            deleted_set = deleted_set
                        )

                try:
                    datastore.commit()
                except ConflictError:
                    datastore.abort()
                    datastore.sync()
                else:
                    break

            # Purge the edit stack of references to deleted items
            if stack:
                prev_stack_size = len(stack)
                stack.remove_references(deleted_set)

            # Launch CMS.item_deleted events
            cms = self.context["cms"]

            for item in deleted_set:
                cms.item_deleted(
                    item = item,
                    user = user,
                    change = item.changes[-1] if item.changes else None
                )

            # If the edit stack has been modified as a result of the delete
            # operation (an edited item has been deleted), redirect the user
            # to the topmost edit node, or to the application root
            if stack is not None and len(stack) < prev_stack_size:

                while len(stack) > 0 \
                and not isinstance(stack[-1], EditNode):
                    stack.pop()

                notify_user(
                    translations(
                        "woost.controllers.DeleteController."
                        "node_deleted_notice"
                    ),
                    category = "error",
                    transient = True
                )

        if stack:
            stack.go()
        else:
            self.go_back()

    view_class = "woost.views.BackOfficeDeleteView"

    @request_property
    def output(self):
        output = BaseBackOfficeController.output(self)
        output["delete_dry_run"] = self.delete_dry_run
        return output




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			September 2010
"""
from woost.models import Document, CreatePermission
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.mailer.mailing import Mailing


class CreateMailingAction(UserAction):

    content_type = Document

    def is_permitted(self, user, target):
        return (
            target.is_inserted
            and user.has_permission(CreatePermission, target = Mailing)
        )

    def get_url(self, controller, selection):
        return controller.contextual_uri(
            'content', 'new',
            item_type = Mailing.full_name,
            edited_item_document = selection[0].id
        )


CreateMailingAction("create_mailing").register()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from cocktail.modeling import cached_getter
from cocktail.controllers import Location
from woost.models import Publishable
from woost.controllers import BaseCMSController


class SitemapController(BaseCMSController):

    namespace = "http://www.sitemaps.org/schemas/sitemap/0.9"

    @cached_getter
    def items(self):
        return Publishable.select_accessible([
            Publishable.sitemap_indexable.equal(True)
        ])

    def __call__(self):

        output = []
        write = output.append

        write(u"<?xml version='1.0' encoding='utf-8'?>")
        write(u"<urlset xmlns='%s'>" % self.namespace)

        base_url = str(Location.get_current_host()).rstrip(u"/")
        uri = self.context["cms"].uri

        for item in self.items:
            write(u"\t<url>")

            item_uri = base_url + u"/" + uri(item).lstrip(u"/")
            write(u"\t\t<loc>%s</loc>" % item_uri)

            date = item.last_update_time.strftime("%Y-%m-%d")
            write(u"\t\t<lastmod>%s</lastmod>" % date)

            frequency = item.sitemap_change_frequency
            if frequency:
                write(u"\t\t<changefreq>%s</changefreq>" % frequency)

            priority = item.sitemap_priority
            if priority:
                write(u"\t\t<priority>%s</priority>" % priority)

            write(u"\t</url>")

        write(u"</urlset>")
        return u"\n".join(output)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail.modeling import OrderedDict
from cocktail.translations import translations, get_language
from cocktail import schema
from cocktail.controllers.location import Location
from woost.models import Extension

translations.define("PaymentsExtension",
    ca = u"Pagaments",
    es = u"Pagos",
    en = u"Payments"
)

translations.define("PaymentsExtension-plural",
    ca = u"Pagaments",
    es = u"Pagos",
    en = u"Payments"
)


class PaymentsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afageix suport per pagaments en línia.""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para pagos en linea.""",
            "es"
        )
        self.set("description",
            u"""Adds support for online payments.""",
            "en"
        )

    def _load(self):

        # Import the extension's models
        from woost.extensions.payments.paymentgateway import PaymentGateway
        from woost.extensions.payments import (
            strings,
            dummypaymentgateway,
            pasat4b,
            sis,
            paypal,
            transactionnotifiedtrigger
        )

        # Setup payment controllers
        from woost.controllers.cmscontroller import CMSController
        from woost.extensions.payments.paymenthandshakecontroller \
            import PaymentHandshakeController
        from woost.extensions.payments.paymentnotificationcontroller \
            import PaymentNotificationController
        from woost.extensions.payments.dummycontroller \
            import DummyPaymentGatewayController

        CMSController.payment_handshake = PaymentHandshakeController
        CMSController.payment_notification = PaymentNotificationController
        CMSController.dummy_payment_gateway = DummyPaymentGatewayController

        # Append additional members to the extension
        PaymentsExtension.members_order = ["payment_gateway"]

        PaymentsExtension.add_member(
            schema.Reference("payment_gateway",
                type = PaymentGateway,
                related_end = schema.Reference()
            )
        )

    def payment_request(self, payment_id):
        """Begin a payment transaction, redirecting the user to the payment
        gateway.

        @param payment_id: The identifier of the payment to execute.
        """
        url, params = self.payment_gateway.get_payment_form_data(
            payment_id,
            get_language()
        )

        location = Location(url)
        location.method = "POST"
        location.form_data = OrderedDict(params)
        location.go()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.controllers import (
    request_property,
    Form,
    FormProcessor
)
from woost.models import get_current_user, ModifyMemberPermission
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.ecommerce import ECommerceExtension
from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder
from woost.extensions.ecommerce.basket import Basket
from woost.extensions.ecommerce.orderstepcontroller import (
    ProceedForm,
    BackForm
)


class CheckoutController(FormProcessor, DocumentController):

    class CheckoutForm(ProceedForm):

        model = Basket.get().get_public_schema()

        @request_property
        def source_instance(self):
            return Basket.get()

        @request_property
        def schema(self):
            schema = ProceedForm.schema(self)
            payment_type = schema.get_member("payment_type")
            if payment_type:
                payment_type.enumeration = \
                    ECommerceExtension.instance.payment_types
            return schema

        def submit(self):
            Form.submit(self)
            Basket.store()
            self.proceed()

    class PreviousStepForm(BackForm):
        pass




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.events import event_handler

from woost.controllers.backoffice.contentcontroller \
    import ContentController

from woost.controllers.backoffice.itemfieldscontroller \
    import ItemFieldsController


class PricingPolicyFieldsController(ItemFieldsController, ContentController):

    @cached_getter
    def persistence_prefix(self):
        return "PricingPolicy%s" % self.stack_node.item.id

    def _handle_form_data(self):
        ItemFieldsController._handle_form_data(self)
        form_data = self.stack_node.form_data
        form_data["matching_items"] = self.get_matching_items_parameters()

    @cached_getter
    def matching_items_parameters_source(self):
        source = self.stack_node.form_data["matching_items"].copy()
        source.update(cherrypy.request.params)
        return source

    def get_matching_items_parameters(self):
        source = self.matching_items_parameters_source
        params = {
            "type": source.get("type"),
            "filter": source.get("filter")
        }
        for key, value in source.iteritems():
            if key.startswith("filter_"):
                params[key] = value

        return params

    @cached_getter
    def user_collection(self):
        user_collection = ContentController.user_collection(self)
        user_collection.__class__.type.clear(user_collection)
        user_collection.params.source = \
            self.matching_items_parameters_source.get
        return user_collection

    @cached_getter
    def output(self):
        output = ContentController.output(self)
        output.update(ItemFieldsController.output(self))
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.controllers.publishablecontroller import PublishableController


class TextFileController(PublishableController):

    def _produce_content(self, **kwargs):
        return self.context["publishable"].content




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""
from __future__ import with_statement
import cherrypy
from cocktail.persistence import datastore
from woost.models import changeset_context, get_current_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.extensions.campaignmonitor import CampaignMonitorExtension
from woost.extensions.campaignmonitor.campaignmonitorlist import \
    CampaignMonitorList


class SyncCampaignMonitorListsController(BaseBackOfficeController):

    view_class = "woost.extensions.campaignmonitor.CampaignMonitorListsSynchronizationView"

    def __call__(self, *args, **kwargs):

        if "cancel" in kwargs:
            raise cherrypy.HTTPRedirect(self.edit_uri(
                CampaignMonitorExtension.instance,
                "lists",
                edit_stack = None
            ))

        return BaseBackOfficeController.__call__(self, *args, **kwargs)

    def submit(self):

        from woost.extensions.campaignmonitor import CampaignMonitorExtension
        extension = CampaignMonitorExtension.instance
        user = get_current_user()

        with changeset_context(author = user) as changeset:
            extension.synchronize_lists(
                restricted = True
            )

            # Report changed lists
            created = set()
            modified = set()
            deleted = set()

            for change in changeset.changes.itervalues():

                if isinstance(change.target, CampaignMonitorList):
                    action_id = change.action.identifier

                    if action_id == "create":
                        created.add(change.target)
                    elif action_id == "modify":
                        modified.add(change.target)
                    elif action_id == "delete":
                        deleted.add(change.target)

        datastore.commit()

        self.output["created_lists"] = created
        self.output["modified_lists"] = modified
        self.output["deleted_lists"] = deleted




#-*- coding: utf-8 -*-
u"""Defines the `Location` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.iteration import first
from cocktail.translations import translations
from cocktail import schema
from woost.models import Item
from woost.controllers.backoffice.contentviews import global_content_views


class Location(Item):

    members_order = [
        "location_name",
        "location_type",
        "code",
        "parent",
        "locations"
    ]

    location_name = schema.String(
        translated = True,
        descriptive = True,
        required = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True
    )

    location_type = schema.String(
        required = True,
        indexed = True,
        enumeration = [
            "continent",
            "country",
            "autonomous_community",
            "province",
            "town"
        ],
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            "" if not value
            else translations(
                "woost.extensions.locations.location_types." + value,
                language,
                **kwargs
            ),
        edit_control = "cocktail.html.DropdownSelector"
    )

    code = schema.String(
        required = True,
        indexed = True,
        text_search = False
    )

    parent = schema.Reference(
        type = "woost.extensions.locations.location.Location",
        bidirectional = True,
        cycles_allowed = False
    )

    locations = schema.Collection(
        items = "woost.extensions.locations.location.Location",
        bidirectional = True,
        cascade_delete = True
    )

    def descend(self, include_self = False):
        """Iterates over the location's descendants.

        :param include_self: If set to True, the location itself is yielded as
            the first element in the iteration.
        :type include_self: bool

        :return: An iterable sequence containing the location's flattened tree.
        :rtype: `Location`
        """
        if include_self:
            yield self
        for location in self.locations:
            for descendant in location.descend(True):
                yield descendant

    def descends_from(self, ascendant):
        """Indicates if the location descends from a given location.

        :param ascendant: The hypothetical container of the location.
        :type ascendant: `Location`

        :return: True if the location is, or is contained within the given
            location or one of its descendants, False otherwise.
        :rtype: bool
        """
        location = self

        while location is not None:
            if location is ascendant:
                return True
            else:
                location = location.parent

        return False

    def get_child_location(self, code):
        """Retrieves the contained location that matches the indicated code.

        :param code: The code of the location to retrieve.
        :type code: str

        :return: The specified location, or None if it wasn't found.
        :rtype: `Location`
        """
        for child in self.locations:
            if child.code == code:
                return child
            for descendant in child.locations:
                match = descendant.get_child_location(code)
                if match:
                    return match

    @classmethod
    def by_code(cls, *code):

        if not code or not code[0]:
            raise ValueError(
                "Location.by_code() requires one or more location codes"
            )

        location = first(cls.select([Location.code.equal(code[0])]))

        for x in code[1:]:
            if location is None:
                return None
            location = location.get_child_location(x)

        return location

    def list_level(self, depth):

        if depth == 1:
            return self.locations
        else:
            descendants = []
            for location in self.locations:
                descendants.extend(location.list_level(depth - 1))
            return descendants


global_content_views.add(
    Location,
    "woost.views.TreeContentView",
    is_default = True,
    inherited = False,
    params = {
        "children_collection": Location.locations
    }
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from pkg_resources import resource_filename


class Application(object):

    __root = None
    __icon_resolver = None

    splash = """\
Made with                     _
__        __ ___   ___   ___ | |_
\ \  __  / // _ \ / _ \ / __||  _|
 \ \/  \/ /| (_) | (_) |___ \| |__
  \__/\__/  \___/ \___/|____/\___/

http://woost.info
"""

    def path(self, *args):
        return os.path.join(self.root, *args)

    def _get_package(self):
        return self.__package

    def _set_package(self, package):
        self.__package = package
        self.root = resource_filename(package, "")

    package = property(_get_package, _set_package)

    def _get_root(self):
        return self.__root

    def _set_root(self, root):

        if self.__root:
            try:
                self.icon_resolver.icon_repositories.remove(
                    self.path("views", "resources", "images", "icons")
                )
            except:
                pass

        self.__root = root

        if root:
            # Add an application specific icon repository
            self.icon_resolver.icon_repositories.insert(
                0, self.path("views", "resources", "images", "icons")
            )

    root = property(_get_root, _set_root)

    @property
    def icon_resolver(self):
        if self.__icon_resolver is None:
            from woost.iconresolver import IconResolver
            self.__icon_resolver = IconResolver()
        return self.__icon_resolver




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from datetime import datetime
from threading import local
from contextlib import contextmanager
from cocktail.modeling import classgetter
from cocktail import schema
from cocktail.translations import translations
from cocktail.persistence import PersistentObject

@contextmanager
def changeset_context(author = None):
    """A context manager that eases item versioning. All changes performed on
    CMS items within the call to the manager will be tracked and made part of a
    L{ChangeSet}.

    Calls to this manager can be nested. In that case, only the topmost call
    will actually produce a new changeset, other calls will reuse the root
    changeset.

    @param author: The author of the changeset.
    @type author: L{User<woost.models.User>}

    Example:

        >>> with changeset_context(some_user) as changeset:
        >>>     item1.price = 15.0
        >>>     item1.stock = 3
        >>>     item2.price = 5.75
        >>>     item3 = MyItem()
        >>> len(changeset.changes)
        3
        >>> changeset.changes[item3.id].action.identifier
        "create"
        >>> item3.author
        some_user
        >>> changeset.changes[item1.id].changed_members
        set(["price", "stock"])
        >>> changeset.changes[item2.id].changed_members
        set(["price"])
        >>> changeset.changes[item2.id].item_state["price"]
        5.75
    """

    changeset = ChangeSet.current

    # Begin a new changeset
    if changeset is None:
        changeset = ChangeSet()
        changeset.author = author
        changeset.begin()
        changeset.insert()

        try:
            yield changeset
        finally:
            changeset.end()

    # Nested call: don't create a new changeset, just reuse the changeset
    # that is already in place
    else:
        if author and author is not changeset.author:
            raise ValueError("A changeset can only have a single author")

        yield changeset


class ChangeSet(PersistentObject):
    """A persistent record of a set of L{changes<Change>} performed on one or
    more CMS items."""

    members_order = "id", "author", "date", "changes"
    indexed = True

    changes = schema.Mapping(searchable = False)

    author = schema.Reference(
        required = True,
        type = "woost.models.User"
    )

    date = schema.DateTime(
        required = True,
        default = schema.DynamicDefault(datetime.now)
    )

    _thread_data = local()

    @classgetter
    def current(cls):
        return getattr(cls._thread_data, "current", None)

    @classgetter
    def current_author(cls):
        cs = cls.current
        return cs and cs.author

    def begin(self):

        if self.current:
            raise TypeError("Can't begin a new changeset, another changeset "
                "is already in place")

        self._thread_data.current = self

    def end(self):
        try:
            del self._thread_data.current
        except AttributeError:
            raise TypeError("Can't finalize the current changeset, there's no "
                "changeset in place")

    def get_searchable_text(self, languages, visited_objects = None):

        if visited_objects is None:
            visited_objects = set()
        elif self in visited_objects:
            return

        visited_objects.add(self)

        # Concatenate the descriptions of change authors and targets
        for language in languages:
            if self.author:
                yield translations(self.author, language)
            for change in self.changes.itervalues():
                yield translations(change.target, language)


class Change(PersistentObject):
    """A persistent record of an action performed on a CMS item."""

    indexed = True

    changeset = schema.Reference(
        required = True,
        type = "woost.models.ChangeSet"
    )

    action = schema.Reference(
        required = True,
        type = "woost.models.Action"
    )

    target = schema.Reference(
        required = True,
        type = "woost.models.Item",
        bidirectional = True
    )

    changed_members = schema.Collection(
        type = set,
        items = schema.String()
    )

    item_state = schema.Mapping(
        required = False
    )

    def __translate__(self, language, **kwargs):
        return translations(
            "woost.models.changesets.Change description",
            action = self.action,
            target = self.target
        ) or PersistentObject.__translate__(self, language, **kwargs)


class ChangeSetHasActionExpression(schema.expressions.Expression):

    def op(self, changeset, action):
        return any(
            change.action is action
            for change in changeset.changes.itervalues()
        )


class ChangeSetHasTargetExpression(schema.expressions.Expression):

    def op(self, changeset, target):
        return any(
            change.target is target
            for change in changeset.changes.itervalues()
        )

    # TODO: optimize queries


class ChangeSetHasTargetTypeExpression(schema.expressions.Expression):

    def op(self, changeset, cls):
        return any(
            isinstance(change.target, cls)
            for change in changeset.changes.itervalues()
        )

    # TODO: optimize queries




#-*- coding: utf-8 -*-
u"""An extension that improves the integration with Facebook of documents
published with Woost.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.html import templates
from cocktail.persistence import datastore
from woost.models import Site, Extension, File


translations.define("OpenGraphExtension",
    ca = u"OpenGraph",
    es = u"OpenGraph",
    en = u"OpenGraph"
)

translations.define("OpenGraphExtension-plural",
    ca = u"OpenGraph",
    es = u"OpenGraph",
    en = u"OpenGraph"
)

translations.define("OpenGraphExtension.open_graph",
    ca = u"Integració amb OpenGraph/Facebook",
    es = u"Integración con OpenGraph/Facebook",
    en = u"OpenGraph/Facebook integration"
)

translations.define("OpenGraphExtension.facebook_administrators",
    ca = u"Comptes de Facebook dels administradors",
    es = u"Cuentas de Facebook de los administradores",
    en = u"Facebook administrator accounts"
)

translations.define("OpenGraphExtension.facebook_applications",
    ca = u"Comptes d'aplicació de Facebook",
    es = u"Cuentas de aplicación de Facebook",
    en = u"Facebook application accounts"
)


class OpenGraphExtension(Extension):

    members_order = [
        "facebook_administrators",
        "facebook_applications"
    ]

    facebook_administrators = schema.String(
        listed_by_default = False,
        searchable = False,
        member_group = "open_graph"
    )

    facebook_applications = schema.String(
        listed_by_default = False,
        searchable = False,
        member_group = "open_graph"
    )

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Millora la integració dels documents del lloc web amb Facebook
            i altres serveis que implementin el protocol OpenGraph""",
            "ca"
        )
        self.set("description",
            u"""Mejora la integración de los documentos del sitio web con
            Facebook y otros servicios que utilicen el protocolo OpenGraph""",
            "es"
        )
        self.set("description",
            u"""Improves the integration of the site's documents with Facebook
            and any other service that makes use of the OpenGraph protocol""",
            "en"
        )

    def _load(self):

        from woost.extensions.opengraph import (
            strings,
            publishable,
            opengraphtype,
            opengraphcategory
        )

        OpenGraphExtension.add_member(
            schema.Collection("categories",
                items = schema.Reference(
                    type = opengraphcategory.OpenGraphCategory
                ),
                related_end = schema.Collection(),
                member_group = "open_graph"
            )
        )

        # Install an overlay to BaseView to automatically add OpenGraph
        # metadata to HTML documents
        templates.get_class("woost.extensions.opengraph.BaseViewOverlay")

        self.install()

    def _install(self):
        self.create_default_categories(verbose = True)

    def create_default_categories(self, verbose = False):

        from woost.extensions.opengraph.opengraphtype import OpenGraphType
        from woost.extensions.opengraph.opengraphcategory \
            import OpenGraphCategory

        for category_id, type_ids in (
            ("activities", (
                "activity",
                "sport"
            )),
            ("businesses", (
                "bar",
                "company",
                "cafe",
                "hotel",
                "restaurant"
            )),
            ("groups", (
                "cause",
                "sports_league",
                "sports_team"
            )),
            ("organizations", (
                "band",
                "government",
                "non_profit",
                "school",
                "university"
            )),
            ("people", (
                "actor",
                "athlete",
                "author",
                "director",
                "musician",
                "politician",
                "profile",
                "public_figure"
            )),
            ("places", (
                "city",
                "country",
                "landmark",
                "state_province"
            )),
            ("products_and_entertainment", (
                "album",
                "book",
                "drink",
                "food",
                "game",
                "movie",
                "product",
                "song",
                "tv_show"
            )),
            ("websites", (
                "article",
                "blog",
                "website"
            ))
        ):
            # Create the category
            og_category = OpenGraphCategory.get_instance(code = category_id)

            if og_category is None:

                if verbose:
                    print "Creating OpenGraph category '%s'" % category_id

                og_category = OpenGraphCategory()
                og_category.code = category_id
                og_category.insert()

                key = "opengraph.categories." + category_id

                for language in translations:
                    label = translations(key, language)
                    if label:
                        og_category.set("title", label, language)

            # Create all types in the category
            types = []

            for type_id in type_ids:
                og_type = OpenGraphType.get_instance(code = type_id)

                if og_type is None:

                    if verbose:
                        print "Creating OpenGraph type '%s.%s'" % (
                            category_id,
                            type_id
                        )

                    og_type = OpenGraphType()
                    og_type.code = type_id
                    og_type.insert()

                    key = "opengraph.types." + type_id

                    for language in translations:
                        label = translations(key, language)
                        if label:
                            og_type.set("title", label, language)

                types.append(og_type)

            og_category.types = types

    def get_global_properties(self):

        site = Site.main

        properties = {}

        if site.site_name:
            properties["og:site_name"] = site.site_name

        if site.logo:
            properties["og:image"] = site.logo.get_uri(host = ".")

        if site.email:
            properties["og:email"] = site.email

        if site.phone_number:
            properties["og:phone_number"] = site.phone_number

        if site.fax_number:
            properties["og:fax_number"] = site.fax_number

        if site.address:
            properties["og:street-address"] = site.address

        if site.town:
            properties["og:locality"] = site.town

        if site.region:
            properties["og:region"] = site.region

        if site.postal_code:
            properties["og:postal-code"] = site.postal_code

        if site.country:
            properties["og:country-name"] = site.country

        if self.facebook_administrators:
            properties["fb:admins"] = self.facebook_administrators

        if self.facebook_applications:
            properties["fb:app_id"] = self.facebook_applications

        return properties

    def get_properties(self, publishable):
        properties = self.get_global_properties()
        properties.update(publishable.get_open_graph_properties())
        return properties




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from cocktail import schema
from cocktail.persistence import PersistentObject

class Action(PersistentObject):

    identifier = schema.String(
        max = 10,
        required = True,
        unique = True,
        indexed = True
    )

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        descriptive = True,
        translated = True
    )

    def __repr__(self):

        base_repr = PersistentObject.__repr__(self)

        if self.identifier:
            base_repr += " (" + self.identifier + ")"

        return base_repr




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from setuptools import setup, find_packages
from os import listdir
from os.path import join, isdir

setup(
    name = "woost",
    version = "1.0.1",
    author = "Whads/Accent SL",
    author_email = "tech@whads.com",
    maintainer = "Marti Congost",
    maintainer_email = "marti.congost@whads.com",
    url = "http://woost.info",
    description = "Model driven multilanguage CMS.",
    long_description =
"""Woost is a Content Management System designed from the ground up  to be able
to publish arbitrary data models, including content in multiple languages. Some
of its features include:\n"

    * Use inheritance, relations and validations to define complex models

    * Automatically generate forms and listings for your models from a schema
      description

    * Translate your content into multiple languages, edit translations side by
      side

    * Compatible with lots of popular Python templating engines

    * Place arbitrarily complex access restrictions

    * Extension framework, with several built-in extensions (comments,
      workflow, integration with online services, and many others)
""",
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: ZODB",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Natural Language :: Catalan",
        "Natural Language :: Spanish",
        "Programming Language :: Python :: 2",
        "Topic :: Internet :: WWW/HTTP :: Site Management"
    ],
    install_requires = [
        "simplejson",
        "cocktail>=1.0,<1.1"
    ],
    extras_require = {
        'campaign_monitor_api': ["campaign_monitor_api"],
        "twitterpublication": ["oauth2"]
    },
    packages = find_packages(),
    include_package_data = True,
    entry_points = {
        "woost.extensions": [
            "workflow = woost.extensions.workflow:WorkflowExtension",
            "shop = woost.extensions.shop:ShopExtension",
            "countries = woost.extensions.countries:CountriesExtension",
            "payments = woost.extensions.payments:PaymentsExtension",
            "comments = woost.extensions.comments:CommentsExtension",
            "recaptcha = woost.extensions.recaptcha:ReCaptchaExtension",
            "staticsite = woost.extensions.staticsite:StaticSiteExtension",
            "sitemap = woost.extensions.sitemap:SitemapExtension",
            "pdf = woost.extensions.pdf:PDFExtension",
            "vimeo = woost.extensions.vimeo:VimeoExtension",
			"signup = woost.extensions.signup:SignUpExtension",
            "googlesearch = woost.extensions.googlesearch:GoogleSearchExtension",
            "googleanalytics = woost.extensions.googleanalytics:GoogleAnalyticsExtension",
            "campaignmonitor = woost.extensions.campaignmonitor:CampaignMonitorExtension",
            "mailer = woost.extensions.mailer:MailerExtension",
            "usermodels = woost.extensions.usermodels:UserModelsExtension",
            "locations = woost.extensions.locations:LocationsExtension",
            "webconsole = woost.extensions.webconsole:WebConsoleExtension",
            "blocks = woost.extensions.blocks:BlocksExtension",
            "opengraph = woost.extensions.opengraph:OpenGraphExtension",
            "ecommerce = woost.extensions.ecommerce:ECommerceExtension",
            "facebookpublication = woost.extensions.facebookpublication:FacebookPublicationExtension",
            "shorturls = woost.extensions.shorturls:ShortURLsExtension",
            "twitterpublication = woost.extensions.twitterpublication:TwitterPublicationExtension",
            "textfile = woost.extensions.textfile:TextFileExtension"
        ]
    },
    # Woost can't yet access view resources (images, style sheets, client
    # side scripts, etc) that are packed inside a zipped egg, so distribution
    # in zipped form is disabled
    zip_safe = False
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.events import event_handler
from cocktail.controllers import (
    request_property,
    get_state,
    get_parameter,
    Pagination,
    session
)
from cocktail.controllers.formprocessor import Form
from woost.models import Publishable
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct
from woost.extensions.ecommerce.productcontroller import ProductController

SESSION_KEY = "woost.extensions.ecommerce.catalog_state"

def catalog_current_state_uri():
    state = session.get(SESSION_KEY) or {}
    catalog = Publishable.require_instance(
        qname = "woost.extensions.ecommerce.catalog_page"
    )
    return catalog.get_uri(parameters = state)


class CatalogController(ProductController):

    @event_handler
    def handle_before_request(cls, event):
        session[SESSION_KEY] = get_state()

    class AddProductForm(ProductController.AddProductForm):

        @request_property
        def product(self):
            return get_parameter(
                schema.Reference(
                    "product",
                    type = ECommerceProduct,
                    required = True
                )
            )

        @request_property
        def model(self):
            return self.product \
                and self.product.purchase_model \
                or Form.model(self)

        @request_property
        def adapter(self):
            return self.product \
                and ProductController.AddProductForm.adapter(self) \
                or Form.adapter(self)

        def create_instance(self):
            return self.product \
                and ProductController.AddProductForm.create_instance(self) \
                or Form.create_instance(self)

    @request_property
    def products(self):
        return ECommerceProduct.select_accessible(order = [
            schema.expressions.CustomExpression(translations)
        ])

    @request_property
    def pagination(self):
        return get_parameter(
            Pagination.copy(**{
                "page_size.default": 20,
                "page_size.max": 100,
                "items": self.products
            }),
            errors = "set_default"
        )

    @request_property
    def output(self):
        output = ProductController.output(self)
        output.update(
            products = self.products,
            pagination = self.pagination
        )
        return output




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
import buffet
from cocktail import schema
from woost.models import Item

class Template(Item):

    members_order = [
        "title",
        "identifier",
        "engine",
        "documents"
    ]

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True
    )

    identifier = schema.String(
        required = True,
        unique = True,
        indexed = True,
        max = 255,
        text_search = False
    )

    engine = schema.String(
        enumeration = buffet.available_engines.keys(),
        text_search = False
    )

    documents = schema.Collection(
        items = "woost.models.Document",
        bidirectional = True,
        editable = False
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Document

Document.add_member(
    schema.Boolean("ga_tracking_enabled",
        default = True,
        required = True,
        listed_by_default = False,
        member_group = "meta"
    )
)

Document.is_ga_tracking_enabled = lambda self: self.ga_tracking_enabled




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import hashlib
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.modeling import cached_getter
from woost.models import User
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.signup import SignUpExtension

def generate_confirmation_hash(email):
    hash = hashlib.sha1()
    hash.update(email)
    hash.update(SignUpExtension.instance.secret_key)
    return hash.hexdigest()

class SignUpConfirmationController(DocumentController):

    autologin = True
    confirmed = False

    def __init__(self, *args, **kwargs):
        self.email = self.params.read(schema.String("email"))
        self.hash = self.params.read(schema.String("hash"))

    def submit(self):

        if self.email or self.hash:
            # Checking hash code
            if generate_confirmation_hash(self.email) == self.hash:
                instance = User.get_instance(email=self.email)
                if instance:
                    # Confirming and enabling user instance
                    instance.set("confirmed_email", True)
                    instance.set("enabled", True)
                    self.confirmed = True
                    datastore.commit()

                    # Autologin after confirmation
                    if self.autologin:
                        self.context["cms"].authentication.set_user_session(instance)

    @cached_getter
    def output(self):
        output = DocumentController.output(self)
        output.update(
            confirmed = self.confirmed
        )

        return output



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from shutil import rmtree
from tempfile import mkdtemp
from subprocess import Popen, PIPE
from time import time, sleep
import Image
from woost.models.file import File
from woost.models.rendering.contentrenderer import ContentRenderer
from woost.models.rendering.contentrenderersregistry import content_renderers


class PDFRenderer(ContentRenderer):
    """A content renderer that handles pdf files."""

    timeout = 20

    try:
        p = Popen(["which", "convert"], stdout=PIPE)
        convert_path = p.communicate()[0].replace("\n", "") or None
    except:
        convert_path = None

    def can_render(self, item):
        return (
            self.convert_path
            and isinstance(item, File)
            and item.resource_type == "document"
            and item.file_name.split(".")[-1].lower() == "pdf"
        )

    def render(self, item, page = 0):

        timeout = self.timeout

        # Increase the timeout for bigger files
        size = item.file_size
        if size:
            timeout += size / (10 * 1024 * 1024)

        RESOLUTION = 0.25
        temp_path = mkdtemp()

        try:
            temp_image_file = os.path.join(temp_path, "thumbnail.png")

            command = u'%s -type TrueColor "%s[%d]" %s' % (
                self.convert_path, item.file_path, page, temp_image_file
            )

            p = Popen(command, shell=True, stdout=PIPE)
            start = time()

            while p.poll() is None:
                if time() - start > timeout:
                    p.terminate()
                    raise IOError("Timeout was reached")
                sleep(RESOLUTION)

            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)

    def last_change_in_appearence(self, item):
        return os.stat(item.file_path).st_mtime


content_renderers.register(PDFRenderer())




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail import schema
from cocktail.events import when
from woost.models.trigger import (
    Trigger,
    trigger_responses
)
from woost.extensions.payments.paymentgateway import PaymentGateway


class TransactionNotifiedTrigger(Trigger):
    instantiable = True


@when(PaymentGateway.transaction_notified)
def launch_transaction_notification_triggers(event):
    trigger_responses(
        TransactionNotifiedTrigger,
        payment = event.payment
    )




#-*- coding: utf-8 -*-
u"""Defines an extension that allows end users to define their own data models.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.translations import translations
from woost.models import Extension

translations.define("UserModelsExtension",
    ca = u"Models d'usuari",
    es = u"Modelos de usuario",
    en = u"User models"
)

translations.define("CountriesExtension-plural",
    ca = u"Models d'usuari",
    es = u"Modelos de usuario",
    en = u"User models"
)

models_access = None


class UserModelsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet als usuaris del gestor definir els seus propis models de
            dades.""",
            "ca"
        )
        self.set("description",
            u"""Permite a los usuarios del gestor definir sus propios modelos
            de datos.""",
            "es"
        )
        self.set("description",
            u"""Makes it possible to create custom data types from the
            backoffice.""",
            "en"
        )

    def _load(self):

        global models_access

        from woost.extensions.usermodels import strings, usermembers, userform
        from woost.extensions.usermodels.door import Door

        # Create extension objects the first time it is run
        self.install()

        # Make all existing user models available to the application
        for user_model in usermembers.UserModel.select():
            user_model.produce_member()

        # Serialize access to models
        models_access = Door()

        cherrypy.request.hooks.attach(
            "on_start_resource",
            models_access.enter,
            failsafe = True
        )

        cherrypy.request.hooks.attach(
            "on_end_request",
            models_access.leave,
            failsafe = True
        )

    def _install(self):

        from woost.models import Controller, Template, extension_translations

        self._create_asset(
            Controller,
            "user_form_controller",
            python_name = "woost.extensions.usermodels.userform."
                          "UserFormController",
            title = extension_translations
        )

        self._create_asset(
            Template,
            "user_form_template",
            identifier = "woost.extensions.usermodels.UserFormView",
            title = extension_translations
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.translations import translations, get_language
from cocktail.html.element import Element
from cocktail.html import templates
from cocktail.html.utils import rendering_xml
from cocktail.controllers import context
from woost.models import (
    Language,
    Site,
    get_current_user,
    ReadTranslationPermission
)

LinkSelector = templates.get_class("cocktail.html.LinkSelector")


class LanguageSelector(LinkSelector):

    translated_labels = True
    tag = "ul"

    missing_translations = "redirect" # "redirect", "hide", "disable"

    def create_entry(self, value, label, selected):

        entry = Element("li")
        link = self.create_entry_link(value, label)

        if selected:
            strong = Element("strong")
            strong.append(link)
            entry.append(strong)
        else:
            entry.append(link)

        if self.missing_translations != "redirect":
            publishable = context["publishable"]
            if not publishable.is_accessible(language = value):
                if self.missing_translations == "hide":
                    entry.visible = False
                elif self.missing_translations == "disable":
                    link.tag = "span"
                    link["href"] = None

        return entry

    def _ready(self):

        if self.items is None:
            user = get_current_user()
            self.items = [
                language.iso_code
                for language in Language.select()
                if language.enabled
                and user.has_permission(
                    ReadTranslationPermission,
                    language = language.iso_code
                )
            ]

        if self.value is None:
            self.value = get_language()

        LinkSelector._ready(self)

        # Hack for IE <= 6
        if self.children:
            self.children[-1].add_class("last")

    def get_item_label(self, language):
        if self.translated_labels:
            return translations(language, language)
        else:
            return translations(language)

    def get_entry_url(self, language):
        cms = context["cms"]
        publishable = context["publishable"]

        if self.missing_translations == "redirect" \
        and not publishable.is_accessible(language = language):
            path = "/"
        else:
            path = None

        return cms.translate_uri(path = path, language = language)

    def create_entry_link(self, value, label):
        link = LinkSelector.create_entry_link(self, value, label)
        link["lang"] = value
        link["hreflang"] = value

        @link.when_ready
        def set_xml_lang():
            if rendering_xml():
                link["xml:lang"] = value

        return link




#-*- coding: utf-8 -*-
u"""
Controllers for the CMS backend and frontend applications.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from warnings import warn
from cocktail.controllers import context
from woost.controllers.cache import get_cache_manager, set_cache_manager
from woost.controllers.cmscontroller import CMSController
from woost.controllers.module import Module
from woost.controllers.basecmscontroller import BaseCMSController
from woost.controllers.notifications import notify_user
from woost.controllers.asyncupload import async_uploader




#-*- coding: utf-8 -*-
u"""
Woost is an object oriented, web based Content Management System, written
in python.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""

__version__ = "1.0.1"

from woost.application import Application
from woost.translations import strings

app = Application()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail.modeling import classgetter
from cocktail.translations import translations, require_language
from cocktail import schema
from woost.models.item import Item


class Language(Item):

    members_order = ["iso_code", "enabled", "fallback"]

    iso_code = schema.String(
        required = True,
        unique = True,
        max = 64,
        text_search = False,
        indexed = True
    )

    enabled = schema.Boolean(
        required = True,
        default = True
    )

    fallback = schema.Reference(
        type = "woost.models.Language",
        bidirectional = True,
        cycles_allowed = False
    )

    fallback_referrers = schema.Collection(
        items = "woost.models.Language",
        bidirectional = True,
        visible = False
    )

    @classgetter
    def codes(cls):
        return [language.iso_code for language in cls.select()]

    def __translate__(self, language, **kwargs):
        return (self.draft_source is None and translations(self.iso_code)) \
            or Item.__translate__(self, language, **kwargs)


def get_common_language(self, members, language = None):

    language = require_language(language)

    if not all(self.get(m, language) for m in members):
        lang = Language.get_instance(iso_code = language)
        if lang is None:
            return lang
        if lang.fallback:
            fallback = lang.fallback.iso_code
            if all(self.get(m, fallback) for m in members):
                language = fallback

    return language

Item.get_common_language = get_common_language
del get_common_language




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2009
"""
from cocktail.modeling import cached_getter
from cocktail import schema
from woost.controllers.backoffice.editstack import EditNode


class UserEditNode(EditNode):

    @cached_getter
    def form_adapter(self):

        form_adapter = EditNode.form_adapter(self)
        form_adapter.exclude("password_confirmation")

        if self.item.is_inserted:
            form_adapter.exclude("change_password")

            if self.item.encryption_method:
                kwargs = {
                    "export_condition": False,
                    "import_condition": lambda context:
                        context.get("change_password", default = None)
                }
            else:
                kwargs = {}

            form_adapter.copy("password", **kwargs)

        return form_adapter

    @cached_getter
    def form_schema(self):

        form_schema = EditNode.form_schema(self)
        password_member = form_schema.get_member("password")

        if password_member:

            if self.item.encryption_method:

                order = form_schema.members_order = list(form_schema.members_order)
                pos = order.index("password")

                password_conf_member = schema.String(
                    name = "password_confirmation",
                    edit_control = "cocktail.html.PasswordBox",
                    visible_in_detail_view = False,
                    required = password_member.required
                )
                form_schema.add_member(password_conf_member)
                order.insert(pos + 1, "password_confirmation")

                if self.item.is_inserted:

                    change_password_member = schema.Boolean(
                        name = "change_password",
                        required = True,
                        default = False,
                        visible_in_detail_view = False
                    )
                    form_schema.add_member(change_password_member)
                    order.insert(pos, "change_password")

                    password_member.exclusive = change_password_member
                    password_conf_member.exclusive = change_password_member

                @form_schema.add_validation
                def validate_password_confirmation(form_schema, value, ctx):
                    password = ctx.get_value("password")
                    password_confirmation = ctx.get_value("password_confirmation")

                    if password and password_confirmation \
                    and password != password_confirmation:
                        yield PasswordConfirmationError(
                                form_schema, value, ctx)

            # No encryption: edit passwords in plain sight
            else:
                password_member.edit_control = "cocktail.html.TextBox"

        return form_schema

    def iter_changes(self):

        # Discard differences in the password field
        for member, language in EditNode.iter_changes(self):
            if not self.item.encryption_method or member.name not in (
                "change_password", "password", "password_confirmation"
            ):
                yield (member, language)


class PasswordConfirmationError(schema.exceptions.ValidationError):
    """A validation error produced when the password and its confirmation field
    are given different values."""




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			September 2010
"""
from cocktail import schema
from woost.models import Permission, Role
from woost.models.messagestyles import permission_doesnt_match_style


class SendEmailPermission(Permission):
    """Permission to send an email."""

    instantiable = True

    roles = schema.Collection(
        items = schema.Reference(
            type = Role,
        ),
        related_end = schema.Collection(),
        relation_constraints = [Role.implicit.equal(False)],
        edit_inline = True
    )

    def match(self, role = None, verbose = False):

        if role and self.roles and role not in self.roles:

            if verbose:
                print permission_doesnt_match_style("Role doesn't match")

            return False

        return Permission.match(self, verbose)




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail import schema
from woost.models import Item


class Comment(Item):

    members_order = ["publishable", "user_name", "user_email", "content"]

    publishable = schema.Reference(
        required = True,
        type = "woost.models.Publishable",
        bidirectional = True,
        related_key = "comments"
    )

    user_name = schema.String(
        required = True
    )

    user_email = schema.String(
        required = True,
        format = r"([\w\-\.]+@(\w[\w\-]+\.)+[\w\-]+)"
    )

    content = schema.String(
        required = True,
        edit_control = "cocktail.html.TextArea"
    )




#-*- coding: utf-8 -*-
u"""Migration of the site's schema.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.persistence import MigrationStep


if __name__ == "__main__":
    import _PROJECT_NAME_
    from cocktail.persistence import migrate
    migrate()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from cocktail import schema
from cocktail.controllers.usercollection import UserCollection
from woost.models.publishable import Publishable
from woost.models.language import Language
from woost.models.controller import Controller


class Feed(Publishable):

    instantiable = True

    groups_order = ["meta", "feed_items"]

    members_order = [
        "title",
        "ttl",
        "image",
        "description",
        "limit",
        "query_parameters",
        "item_title_expression",
        "item_link_expression",
        "item_publication_date_expression",
        "item_description_expression"
    ]

    default_mime_type = u"application/rss+xml"

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.feed_controller")
    )

    edit_controller = \
        "woost.controllers.backoffice.feedfieldscontroller." \
        "FeedFieldsController"
    edit_view = "woost.views.FeedFields"

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        member_group = "meta"
    )

    ttl = schema.Integer(
        listed_by_default = False,
        member_group = "meta"
    )

    image = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        relation_constraints = Publishable.resource_type.equal("image"),
        member_group = "meta"
    )

    description = schema.String(
        required = True,
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    limit = schema.Integer(
        min = 1,
        listed_by_default = False,
        member_group = "feed_items"
    )

    query_parameters = schema.Mapping(
        keys = schema.String(),
        required = True,
        listed_by_default = False,
        member_group = "feed_items"
    )

    item_title_expression = schema.CodeBlock(
        language = "python",
        required = True,
        default = "translations(item)",
        member_group = "feed_items"
    )

    item_link_expression = schema.CodeBlock(
        language = "python",
        required = True,
        default = "cms.uri(item)",
        member_group = "feed_items"
    )

    item_publication_date_expression = schema.CodeBlock(
        language = "python",
        required = True,
        default = "item.start_date or item.creation_time",
        member_group = "feed_items"
    )

    item_description_expression = schema.CodeBlock(
        language = "python",
        required = True,
        default = "item.description",
        member_group = "feed_items"
    )

    def select_items(self):

        user_collection = UserCollection(Publishable)
        user_collection.allow_paging = False
        user_collection.allow_member_selection = False
        user_collection.allow_language_selection = False
        user_collection.params.source = self.query_parameters.get
        user_collection.available_languages = Language.codes
        items = user_collection.subset

        if self.limit:
            items.range = (0, self.limit)

        return items




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import re
from cocktail import schema
from woost.models import Item


class Country(Item):

    members_order = ["country_name", "iso_code"]

    country_name = schema.String(
        unique = True,
        required = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True
    )

    iso_code = schema.String(
        unique = True,
        required = True,
        indexed = True,
        format = re.compile(r"^[a-z]{2}$"),
        text_search = False
    )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from cocktail.events import Event
from cocktail import schema
from woost.models import Item


class Transition(Item):
    """A transition between two item states."""

    visible_from_root = False

    executed = Event("""An event triggered when the transition is executed.

        @var item: The item that the transition is executed on.
        @type item: L{Item<woost.models.item.Item>}

        @var data: Additional data supplied to the transition, as required by
            its L{schema<transition_form>}.
        @type data: dict
        """)

    members_order = [
        "title",
        "source_state",
        "target_state",
        "transition_form",
        "transition_permissions"
    ]

    title = schema.String(
        required = True,
        descriptive = True,
        translated = True
    )

    source_state = schema.Reference(
        type = "woost.extensions.workflow.state.State",
        related_key = "outgoing_transitions",
        bidirectional = True,
        edit_control = "cocktail.html.DropdownSelector"
    )

    target_state = schema.Reference(
        type = "woost.extensions.workflow.state.State",
        related_key = "incoming_transitions",
        bidirectional = True,
        edit_control = "cocktail.html.DropdownSelector"
    )

    transition_form = schema.String(
        text_search = False
    )

    transition_permissions = schema.Collection(
        items = "woost.extensions.workflow.transitionpermission."
                "TransitionPermission",
        related_key = "transition",
        bidirectional = True
    )

    def execute(self, item, data = None):

        if item.workflow_state is not self.source_state:
            raise ValueError(
                "Can't execute the transition on the indicated item"
            )

        item.workflow_state = self.target_state
        self.executed(item = item, data = data)



#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.events import when
from cocktail.modeling import underscore_to_capital
from cocktail.translations import (
    translations,
    set_language,
    language_context
)
from cocktail import schema
from cocktail.persistence import datastore
from woost.models import (
    Site,
    Extension,
    Publishable,
    Document,
    StandardPage,
    Template,
    Controller,
    EmailTemplate,
    User,
    Language
)
from woost.models.triggerresponse import SendEmailTriggerResponse

translations.define("ECommerceExtension",
    ca = u"Comerç online",
    es = u"Comercio online",
    en = u"E-commerce"
)

translations.define("ECommerceExtension-plural",
    ca = u"Comerç online",
    es = u"Comercio online",
    en = u"E-commerce"
)


class ECommerceExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet vendre productes des del lloc web.""",
            "ca"
        )
        self.set("description",
            u"""Permite vender productos des del sitio web.""",
            "es"
        )
        self.set("description",
            u"""Allows selling products online.""",
            "en"
        )

    def _load(self):

        extension = self

        from woost.extensions.ecommerce import (
            strings,
            ecommerceproduct,
            ecommerceorder,
            ecommercepurchase,
            ecommercebillingconcept,
            ecommerceordercompletedtrigger,
            imagefactories
        )
        from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder

        # Add the necessary members to define pricing policies
        ECommerceExtension.members_order = [
            "payment_types",
            "pricing",
            "shipping_costs",
            "taxes",
            "order_steps"
        ]

        available_payment_types = ("payment_gateway", "transfer", "cash_on_delivery")
        ECommerceOrder.payment_type.enumeration = available_payment_types

        ECommerceExtension.add_member(
            schema.Collection(
                "payment_types",
                items = schema.String(
                    enumeration = available_payment_types,
                    translate_value = lambda value, language = None, **kwargs:
                        translations(
                            "ECommerceExtension.payment_types-%s" % value,
                            language = language
                        ),
                ),
                min = 1
            )
        )

        ECommerceExtension.add_member(
            schema.Collection("pricing",
                items = schema.Reference(
                    type = ecommercebillingconcept.ECommerceBillingConcept
                ),
                bidirectional = True,
                related_end = schema.Collection(),
                integral = True
            )
        )

        ECommerceExtension.add_member(
            schema.Collection("shipping_costs",
                items = schema.Reference(
                    type = ecommercebillingconcept.ECommerceBillingConcept
                ),
                bidirectional = True,
                related_end = schema.Collection(),
                integral = True
            )
        )

        ECommerceExtension.add_member(
            schema.Collection("taxes",
                items = schema.Reference(
                    type = ecommercebillingconcept.ECommerceBillingConcept
                ),
                bidirectional = True,
                related_end = schema.Collection(),
                integral = True
            )
        )

        ECommerceExtension.add_member(
            schema.Collection("order_steps",
                items = schema.Reference(type = Publishable),
                related_end = schema.Collection()
            )
        )

        # If the payments extension is enabled, setup the payment gateway for
        # the shop
        from woost.extensions.payments import PaymentsExtension
        payments_ext = PaymentsExtension.instance

        if payments_ext.enabled:
            extension._setup_payment_gateway()

        # Create the pages for the shop the first time the extension runs
        self.install()

    def _setup_payment_gateway(self):

        from tpv import (
            Currency,
            Payment,
            PaymentItem,
            PaymentNotFoundError
        )
        from woost.extensions.payments.paymentgateway import PaymentGateway
        from woost.extensions.payments.transactionnotifiedtrigger \
            import launch_transaction_notification_triggers
        from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder
        from woost.extensions.payments import PaymentsExtension

        payments_ext = PaymentsExtension.instance

        def get_payment(self, payment_id):

            order = ECommerceOrder.get_instance(int(payment_id))

            if order is None:
                raise PaymentNotFoundError(payment_id)

            payment = Payment()
            payment.id = order.id
            payment.description = order.get_description_for_gateway()
            payment.amount = order.total
            payment.order = order
            payment.currency = Currency(payments_ext.payment_gateway.currency)

            for purchase in order.purchases:
                payment.add(PaymentItem(
                    reference = str(purchase.product.id),
                    description = translations(purchase.product),
                    units = purchase.quantity,
                    price = purchase.total
                ))

            return payment

        PaymentGateway.get_payment = get_payment

        def receive_order_payment(event):
            payment = event.payment
            order = payment.order
            set_language(order.language)
            order.status = payment.status
            order.gateway_parameters = payment.gateway_parameters

        def commit_order_payment(event):
            datastore.commit()

        events = PaymentGateway.transaction_notified
        pos = events.index(launch_transaction_notification_triggers)
        events.insert(pos, receive_order_payment)
        events.insert(pos + 2, commit_order_payment)

    def _install(self):

        catalog = self._create_document("catalog")
        catalog.controller = self._create_controller("catalog")
        catalog.template = self._create_template("catalog")
        catalog.parent = Site.main.home
        catalog.insert()

        for child_name in (
            "basket",
            "checkout",
            "summary"
        ):
            child = self._create_document(child_name)
            child.hidden = True
            child.template = self._create_template(child_name)
            child.controller = self._create_controller(child_name)
            child.parent = catalog
            child.insert()
            self.order_steps.append(child)

        for child_name in (
            "success",
            "failure"
        ):
            child = self._create_document(child_name, StandardPage)
            child.hidden = True
            child.parent = catalog
            child.insert()

        self._create_controller("product").insert()
        self._create_template("product").insert()
        self._create_ecommerceorder_completed_trigger().insert()
        self._create_incoming_order_trigger().insert()

    def _create_document(self, name,
        cls = Document,
        template = None,
        controller = None):

        document = cls()
        document.qname = "woost.extensions.ecommerce.%s_page" % name

        self.__translate_field(document, "title")

        if isinstance(document, StandardPage):
            self.__translate_field(document, "body")

        return document

    def _create_template(self, name):
        template = Template()
        template.qname = "woost.extensions.ecommerce.%s_template" % name
        self.__translate_field(template, "title")
        template.engine = "cocktail"
        template.identifier = \
            "woost.extensions.ecommerce.%sPage" % underscore_to_capital(name)
        return template

    def _create_controller(self, name):
        controller = Controller( )
        controller.qname = "woost.extensions.ecommerce.%s_controller" % name
        self.__translate_field(controller, "title")
        controller.python_name = \
            "woost.extensions.ecommerce.%scontroller.%sController" % (
                name,
                underscore_to_capital(name)
            )
        return controller

    def _create_ecommerceorder_completed_trigger(self):
        from woost.extensions.ecommerce.ecommerceordercompletedtrigger import \
            ECommerceOrderCompletedTrigger
        trigger = ECommerceOrderCompletedTrigger( )
        trigger.qname = \
            "woost.extensions.ecommerce.ecommerceorder_completed_trigger"
        self.__translate_field(trigger, "title")
        trigger.site = Site.main
        trigger.condition = "target.customer and not target.customer.anonymous and target.customer.email"
        trigger.matching_items = {'type': u'woost.extensions.ecommerce.ecommerceorder.ECommerceOrder'}

        # EmailTemplate
        template = EmailTemplate()
        template.qname = \
            "woost.extensions.ecommerce.ecommerceorder_completed_emailtemplate"
        self.__translate_field(template, "title")
        template.sender = '"%s"' % User.require_instance(
            qname = "woost.administrator"
        ).email
        template.receivers = '[items[0].customer.email]'
        template.embeded_images = """
from woost.models import Site
images["logo"] = Site.main.logo
"""
        template.template_engine = "cocktail"

        for language in translations:
            with language_context(language):
                template.subject = template.title
                template.body = """
<%
from cocktail.controllers import Location
from cocktail.html import templates

order = items[0]
order_summary = templates.new("woost.extensions.ecommerce.OrderConfirmationMessage")
order_summary.order = order
%>

<html>
    <head>
        <base href="@{unicode(Location.get_current_host())}"/>
    </head>
    <body>
        ${order_summary.render()}
    </body>
</html>
"""

        # Response
        response = SendEmailTriggerResponse()
        response.qname = \
            "woost.extensions.ecommerce.ecommerceorder_completed_response"
        response.email_template = template

        trigger.responses = [response]
        return trigger

    def _create_incoming_order_trigger(self):
        from woost.extensions.ecommerce.incomingordertrigger import \
            IncomingOrderTrigger
        trigger = IncomingOrderTrigger( )
        trigger.qname = "woost.extensions.ecommerce.incoming_order.trigger"
        self.__translate_field(trigger, "title")
        trigger.site = Site.main
        trigger.matching_items = {'type': u'woost.extensions.ecommerce.ecommerceorder.ECommerceOrder'}

        # EmailTemplate
        template = EmailTemplate()
        template.qname = \
            "woost.extensions.ecommerce.incoming_order.email_template"
        self.__translate_field(template, "title")
        admin = User.require_instance(qname = "woost.administrator")
        template.sender = repr(admin.email)
        template.receivers = repr([admin.email])
        template.template_engine = "cocktail"

        for language in translations:
            with language_context(language):
                template.subject = template.title
                template.body = """
<%
from cocktail.translations import translations
from woost.models import Publishable

order = items[0]
bo = Publishable.require_instance(qname = "woost.backoffice")
edit_url = bo.get_uri(host = ".", path = ["content", str(order.id)])
%>

<html>
    <body>
        <a href="${edit_url}">${translations('woost.extensions.ecommerce.incoming_order.edit_link')}
        </a>
    </body>
</html>
"""

        # Response
        response = SendEmailTriggerResponse()
        response.email_template = template
        trigger.responses = [response]

        return trigger

    def __translate_field(self, obj, key):
        for language in translations:
            with language_context(language):
                value = translations("%s.%s" % (obj.qname, key))
                if value:
                    obj.set(key, value)




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
from cocktail.html import templates, Element
from cocktail.controllers import context
from woost.models import Site

TreeView = templates.get_class("cocktail.html.TreeView")


class Menu(TreeView):
    """A visual component used to render navigation menus and site maps.

    @var emphasized_selection: When set to True, adds a <strong> tag to
        highlight the selected item.
    @type emphasized_selection: bool

    @var linked_selection: Indicates if the entry for the selected item should
        behave as a link or not.
    @type linked_selection: bool

    @var linked_containers: Indicates if entries that contain other entries
        should behave as links.
    @type linked_containers: bool
    """
    root_visibility = TreeView.HIDDEN_ROOT
    expanded = False
    emphasized_selection = True
    linked_selection = True
    linked_containers = True
    display_filtered_containers = False
    validate_permissions = True

    def _ready(self):

        if self.root is None:
            self.root = Site.main.home

        if self.selection is None:
            self.selection = context["publishable"]

        TreeView._ready(self)

    def filter_item(self, item):
        return not item.hidden and (
            item.is_accessible()
            if self.validate_permissions
            else item.is_published()
        )

    def get_item_uri(self, item):
        return item.get_uri(host = ".")

    def create_label(self, item):

        if self.should_link(item):
            label = Element("a")
            label["href"] = self.get_item_uri(item)
        else:
            label = Element("span")

        label.append(self.get_item_label(item))

        if self.emphasized_selection and item is self.selection:
            if label.tag == "a":
                label = Element("strong", children = [label])
            else:
                label.tag = "strong"

        return label

    def should_link(self, item):
        return (self.linked_selection or item is not self.selection) \
            and (self.linked_containers or not item.children)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.modeling import extend, call_base
from cocktail.translations import translations
from cocktail.schema import Member, Reference, Collection
from cocktail.html import templates, Element
from woost.models import (
    get_current_user,
    Item,
    ReadMemberPermission
)


class ContentDisplayMixin(object):

    base_url = None
    referer = None

    def __init__(self):

        @extend(self)
        def get_default_member_display(self, obj, member):

            if isinstance(member, Reference):
                if member.is_persistent_relation \
                and issubclass(member.type, Item):
                    return self.display_item_reference

            if isinstance(member, Collection):
                if member.is_persistent_relation \
                and issubclass(member.items.type, Item):
                    return self.display_item_collection

            return call_base(obj, member)

    def display_item_reference(self, obj, member):
        display = templates.new("woost.views.ContentLink")
        display.item = self.get_member_value(obj, member)
        display.base_url = self.base_url
        display.referer = obj
        display.member = member
        return display

    def display_item_collection(self, obj, member):
        display = templates.new("woost.views.ContentList")
        display.items = self.get_member_value(obj, member)
        display.base_url = self.base_url
        display.referer = obj
        display.member = member
        return display




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2009
"""
from __future__ import with_statement
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.events import event_handler
from cocktail.pkgutils import import_object
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import (
    get_parameter,
    FormControllerMixin
)
from cocktail.controllers.viewstate import view_state
from woost.models import (
    Item,
    get_current_user,
    ReadPermission,
    ModifyPermission
)
from woost.models.changesets import changeset_context
from woost.controllers import notify_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.controllers.backoffice.itemfieldscontroller \
    import ItemFieldsController
from woost.controllers.backoffice.collectioncontroller \
    import CollectionController
from woost.extensions.workflow.transition import Transition
from woost.extensions.workflow.transitionpermission \
    import TransitionPermission


class TransitionController(FormControllerMixin, BaseBackOfficeController):

    view_class = "woost.extensions.workflow.TransitionView"

    @event_handler
    def handle_traversed(cls, event):
        controller = event.source
        get_current_user().require_permission(
            TransitionPermission,
            target = controller.item,
            transition = controller.transition
        )

    @cached_getter
    def item(self):

        item = get_parameter(schema.Reference("item", type = Item))

        if item is None or item.workflow_state is None:
            raise ValueError("Invalid item selected")

        return item

    @cached_getter
    def transition(self):

        transition = get_parameter(
            schema.Reference("transition", type = Transition)
        )

        if transition is None \
        or transition not in self.item.workflow_state.outgoing_transitions:
            raise ValueError("Invalid transition selected")

        return transition

    @cached_getter
    def form_model(self):
        form_model =  import_object(self.transition.transition_form)

        if isinstance(form_model, schema.Schema):
            return form_model
        elif callable(form_model):
            form_model = form_model(self.item, self.transition)
            if isinstance(form_model, schema.Schema):
                return form_model

        raise ValueError("Invalid transition form")

    @cached_getter
    def form_errors(self):

        if "cancel" in cherrypy.request.params:
           return schema.ErrorList([])
        else:
            return FormControllerMixin.form_errors(self)

    @cached_getter
    def form_validation_context(self):
        context = FormControllerMixin.form_validation_context(self)
        context.update(
            workflow_item = self.item,
            workflow_transition = self.transition
        )
        return context

    def submit(self):

        item = self.item
        draft_source = item.draft_source

        if cherrypy.request.method == "POST" \
        and not "cancel" in cherrypy.request.params:

            # Apply form data
            FormControllerMixin.submit(self)

            transition = self.transition

            # Authorization check
            user = get_current_user()
            user.require_permission(
                TransitionPermission,
                target = item,
                transition = transition
            )

            # Execute the transition
            with changeset_context(user):
                transition.execute(item, data = self.form_instance)
                datastore.commit()

            # Inform the user of the result
            notify_user(
                translations(
                    "woost.controllers.backoffice.useractions.TransitionAction"
                    " state set",
                    item = item
                ),
                "success"
            )

        # Redirect the user to the transitioned item's edit form
        redirect_transition(item if item.is_inserted else draft_source)

    @cached_getter
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            item = self.item,
            transition = self.transition
        )
        return output

def redirect_transition(item):
    """ GET redirection, to avoid duplicate POST requests. Also, permissions
    that allowed the user to edit or view the transitioned item may no
    longer apply (as they may have been granted by the item's previous
    state). If that is the case, redirect the user to the best available
    view """

    user = get_current_user()
    controller = cherrypy.request.handler_chain[-1]
    cms = controller.context["cms"]
    url = None

    if isinstance(
        controller,
        (ItemFieldsController, CollectionController, TransitionController)
    ):
        if not user.has_permission(ModifyPermission, target = item):
            if user.has_permission(ReadPermission, target = item):
                url = controller.edit_uri(item, "show_detail")
            else:
                url = cms.contextual_uri()
        elif isinstance(controller, TransitionController):
            if controller.edit_stack:
                controller.go_back()
            else:
                url = controller.edit_uri(item, "show_detail")

    elif not user.has_permission(ReadPermission, target = item):
        url = cms.contextual_uri()

    raise cherrypy.HTTPRedirect(
        url or "?" + view_state(item_action = None, transition = None)
    )




#!/usr/bin/env python
#-*- coding: utf-8 -*-
u"""A convenience script that can be launched through ipython or vanilla python
with the -i flag, to have quick access to the project's data.
"""
from __future__ import with_statement
from cocktail.translations import *
from cocktail.persistence import *
from woost import app
from woost.models import *
from woost.models.extension import load_extensions
from _PROJECT_MODULE_.models import *

site = Site.main

DEFAULT_LANGUAGE = site.default_language
set_language(DEFAULT_LANGUAGE)
load_extensions()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2009
"""
from cocktail import schema
from cocktail.modeling import cached_getter
from cocktail.translations import get_language
from woost.controllers.backoffice.editcontroller import EditController
from woost.controllers.backoffice.useractions import get_user_action


class PreviewController(EditController):

    section = "preview"

    def __init__(self, *args, **kwargs):
        EditController.__init__(self, *args, **kwargs)
        self.preview_language = self.params.read(
            schema.String("preview_language", default = get_language())
        )

    @cached_getter
    def view_class(self):
        return self.stack_node.item.preview_view

    @cached_getter
    def output(self):

        # TODO: Add a translation selector

        output = EditController.output(self)
        output.update(
            selected_action = get_user_action("preview"),
            preview_language = self.preview_language
        )

        return output




#-*- coding: utf-8 -*-
u"""Defines the `LoginController` class.

.. moduleauthor:: Javier Marrero <javier.marrero@whads.com>
"""
import cherrypy
from cocktail import schema
from cocktail.modeling import cached_getter
from cocktail.events import Event, event_handler
from cocktail.schema.exceptions import ValidationError
from cocktail.controllers.parameters import get_parameter
from cocktail.controllers.formcontrollermixin import FormControllerMixin
from woost.models import (
    User,
    AuthorizationError,
    get_current_user
)
from woost.controllers.documentcontroller import DocumentController
from woost.controllers.authentication import AuthenticationFailedError


class LoginController(FormControllerMixin, DocumentController):

    def __init__(self, *args, **kwargs):
        DocumentController.__init__(self, *args, **kwargs)
        FormControllerMixin.__init__(self)

    @cached_getter
    def form_errors(self):
        form_errors = FormControllerMixin.form_errors(self)

        if not form_errors and self.submitted and get_current_user().anonymous:
            user_param = get_parameter(schema.Member("user"))
            form_errors._items.append(AuthenticationFailedError(user_param))

        return form_errors

    @cached_getter
    def submitted(self):
        return cherrypy.request.method == "POST" and \
            get_parameter(schema.Member("authenticate"))

    @cached_getter
    def form_model(self):

        identifier = self.context["cms"].authentication.identifier_field.copy()
        identifier.name = "user"

        form_model = schema.Schema("woost.controllers.logincontroller.form",
            members = [
                identifier,
                User.password.copy()
            ]
        )

        return form_model




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail import schema
from woost.models import Publishable, Controller


class Product(Publishable):

    instantiable = False
    view_class = None

    members_order = [
        "price",
        "categories",
        "entries"
    ]

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.product_controller")
    )

    price = schema.Decimal(
        required = True,
        default = Decimal("0")
    )

    categories = schema.Collection(
        items = "woost.extensions.shop.productcategory.ProductCategory",
        bidirectional = True
    )

    entries = schema.Collection(
        items = "woost.extensions.shop.shoporderentry.ShopOrderEntry",
        bidirectional = True,
        visible = False,
        block_delete = True
    )

    def discounts(self):
        """Returns the discounts that can be applied to the product.
        @rtype: L{Product<woost.extensions.shop.product.Product>}
        """
        from woost.extensions.shop import ShopExtension
        return [discount
                for discount in ShopExtension.instance.discounts
                if discount.applies_to(self)]




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from woost.models import Extension

translations.define("TextFileExtension",
    ca = u"Fitxers de text",
    es = u"Ficheros de texto",
    en = u"Text files"
)


class TextFileExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"Permet editar fitxers de text directament des del gestor.",
            "ca"
        )
        self.set("description",
            u"Permite editar ficheros de texto directamente desde el gestor.",
            "es"
        )
        self.set("description",
            u"Makes it possible to edit text files from the backoffice.",
            "en"
        )

    def _load(self):
        from woost.extensions.textfile import textfile, strings
        self.install()

    def _install(self):

        from woost.models import Controller, extension_translations

        controller = self._create_asset(
            Controller,
            "controller",
            title = extension_translations,
            python_name = "woost.extensions.textfile.textfilecontroller."
                          "TextFileController"
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from __future__ import with_statement
from woost.tests.models.basetestcase import BaseTestCase
from nose.tools import assert_raises


class PermissionTestCase(BaseTestCase):

    def assert_authorized(self, user, permission_type, **context):
        assert user.has_permission(permission_type, **context)
        user.require_permission(permission_type, **context)

    def assert_not_authorized(self, user, permission_type, **context):
        from woost.models import AuthorizationError
        assert not user.has_permission(permission_type, **context)
        assert_raises(
            AuthorizationError,
            user.require_permission,
            permission_type,
            **context
        )

    def test_permission_acquisition(self):

        from woost.models import (
            User,
            Role,
            ReadPermission
        )

        R = lambda **kwargs: Role(permissions = [ReadPermission()], **kwargs)
        r1 = R()
        r2 = R()
        r3 = R()
        r4 = R(base_roles = [r3])
        r5 = R(base_roles = [r2, r4])

        self.everybody_role.permissions.append(ReadPermission())
        self.authenticated_role.permissions.append(ReadPermission())

        user = User()
        user.roles = [r1, r5]

        print list(user.iter_permissions())

        assert list(user.iter_permissions()) == [
            r1.permissions[0],
            r5.permissions[0],
            r2.permissions[0],
            r4.permissions[0],
            r3.permissions[0],
            self.authenticated_role.permissions[0],
            self.everybody_role.permissions[0]
        ]

    def test_content_permissions(self):

        from woost.models import (
            Item,
            Publishable,
            User,
            Role,
            Permission,
            CreatePermission,
            ReadPermission,
            ModifyPermission,
            DeletePermission
        )

        class TestPermission(Permission):
            pass

        item = Item()
        doc = Publishable()

        for permission_type in [
            CreatePermission,
            ReadPermission,
            ModifyPermission,
            DeletePermission
        ]:
            role = Role()
            user = User(roles = [role])

            # Permission denied (no permissions defined)
            self.assert_not_authorized(user, permission_type, target = doc)

            # Permission granted
            role.permissions.append(
                permission_type(
                    matching_items = {
                        "type": "woost.models.publishable.Publishable"
                    },
                    authorized = True
                )
            )

            self.assert_authorized(user, permission_type, target = doc)

            # Permission denied (wrong target)
            self.assert_not_authorized(user, permission_type, target = item)

            # Permission denied (wrong permission type)
            self.assert_not_authorized(user, TestPermission)

            # Permission denied (prevailing negative permission)
            role.permissions.insert(
                0,
                permission_type(
                    matching_items = {
                        "type": "woost.models.item.Item"
                    },
                    authorized = False
                )
            )

            self.assert_not_authorized(user, permission_type, target = doc)

    def test_member_permissions(self):

        from woost.models import (
            Item,
            Publishable,
            User,
            Role,
            Permission,
            ReadMemberPermission,
            ModifyMemberPermission
        )

        class TestPermission(Permission):
            pass

        prefix = "woost.models.publishable.Publishable."
        m1 = Publishable.resource_type
        m2 = Publishable.parent
        m3 = Publishable.controller

        for permission_type in [ReadMemberPermission, ModifyMemberPermission]:

            role = Role()
            user = User(roles = [role])

            # No permissions by default
            self.assert_not_authorized(user, permission_type, member = m1)
            self.assert_not_authorized(user, permission_type, member = m2)
            self.assert_not_authorized(user, permission_type, member = m3)

            # Positive permission
            role.permissions.append(
                permission_type(
                    matching_members = [
                        prefix + m1.name,
                        prefix + m2.name
                    ],
                    authorized = True
                )
            )

            self.assert_authorized(user, permission_type, member = m1)
            self.assert_authorized(user, permission_type, member = m2)
            self.assert_not_authorized(user, permission_type, member = m3)

            # Permission denied (wrong permission type)
            self.assert_not_authorized(user, TestPermission)

            # Negative permission
            role.permissions.insert(
                0,
                permission_type(
                    matching_members = [prefix + m1.name],
                    authorized = False
                )
            )

            self.assert_not_authorized(user, permission_type, member = m1)
            self.assert_authorized(user, permission_type, member = m2)
            self.assert_not_authorized(user, permission_type, member = m3)

    def test_translation_permissions(self):

        from woost.models import (
            Item,
            Publishable,
            User,
            Role,
            Permission,
            CreateTranslationPermission,
            ReadTranslationPermission,
            ModifyTranslationPermission,
            DeleteTranslationPermission
        )

        class TestPermission(Permission):
            pass

        l1 = "en"
        l2 = "fr"
        l3 = "ru"

        for permission_type in [
            CreateTranslationPermission,
            ReadTranslationPermission,
            ModifyTranslationPermission,
            DeleteTranslationPermission
        ]:
            role = Role()
            user = User(roles = [role])

            # No permissions by default
            self.assert_not_authorized(user, permission_type, language = l1)
            self.assert_not_authorized(user, permission_type, language = l2)
            self.assert_not_authorized(user, permission_type, language = l3)

            # Permission granted
            role.permissions.append(
                permission_type(
                    matching_languages = [l1, l2],
                    authorized = True
                )
            )

            self.assert_authorized(user, permission_type, language = l1)
            self.assert_authorized(user, permission_type, language = l2)
            self.assert_not_authorized(user, permission_type, language = l3)

            # Permission denied (wrong permission type)
            self.assert_not_authorized(user, TestPermission)

            # Negative permission
            role.permissions.insert(
                0,
                permission_type(
                    matching_languages = [l1],
                    authorized = False
                )
            )

            self.assert_not_authorized(user, permission_type, language = l1)
            self.assert_authorized(user, permission_type, language = l2)
            self.assert_not_authorized(user, permission_type, language = l3)

    def test_permission_expression(self):

        from woost.models import (
            Item,
            Publishable,
            User,
            ReadPermission,
            PermissionExpression
        )

        self.everybody_role.permissions.append(
            ReadPermission(
                matching_items = {
                    "type": "woost.models.publishable.Publishable"
                },
                authorized = True
            )
        )

        i1 = Item()
        i2 = Item()
        d1 = Publishable()
        d2 = Publishable()

        i1.insert()
        i2.insert()
        d1.insert()
        d2.insert()

        results = set(Item.select(filters = [
            PermissionExpression(User(), ReadPermission)
        ]))

        assert results == set([d1, d2])




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2009
"""
from cocktail.modeling import cached_getter
from cocktail.schema import Adapter, RelationMember
from woost.models import Language
from woost.controllers.backoffice.editcontroller import EditController
from woost.controllers.backoffice.useractions import get_user_action


class ShowDetailController(EditController):

    section = "show_detail"

    @cached_getter
    def detail_schema_adapter(self):
        adapter = Adapter()
        adapter.exclude([
            member.name
            for member in self.stack_node.content_type.members().itervalues()
            if not member.visible
            or not member.visible_in_detail_view
            or (
                isinstance(member, RelationMember)
                and member.is_persistent_relation
                and member.related_type
                and not member.related_type.visible
                and not member.related_type.visible_in_detail_view
            )
        ])
        adapter.exclude(["translations"])
        return adapter

    @cached_getter
    def detail_schema(self):
        content_type = self.stack_node.content_type
        adapter = self.detail_schema_adapter
        schema = adapter.export_schema(content_type)
        schema.name = content_type.name + "Detail"
        return schema

    @cached_getter
    def view_class(self):
        return self.stack_node.item.show_detail_view

    @cached_getter
    def output(self):

        # TODO: Add a translation selector

        output = EditController.output(self)
        output.update(
            translations = Language.codes,
            detail_schema = self.detail_schema,
            selected_action = get_user_action("show_detail")
        )

        node = self.stack_node
        node.import_form_data(node.form_data, node.item)

        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import cherrypy
from cocktail.modeling import extend, call_base
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers import (
    get_parameter,
    request_property,
    FormProcessor,
    Form,
    Location
)
from woost.models import Publishable
from woost.controllers.documentcontroller import DocumentController
from woost.controllers.notifications import notify_user
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct
from woost.extensions.ecommerce.ecommercepurchase import ECommercePurchase
from woost.extensions.ecommerce.basket import Basket
from woost.extensions.ecommerce.orderstepcontroller import (
    ProceedForm,
    BackForm
)


class BasketController(FormProcessor, DocumentController):

    @request_property
    def action(self):

        delete_purchase = cherrypy.request.params.get("delete_purchase", None)

        if delete_purchase:
            cherrypy.request.params["purchase"] = delete_purchase
            return "delete_purchase"

        return FormProcessor.action(self)

    class SetQuantitiesForm(Form):
        """Update the number of units for each purchase."""

        actions = "set_quantities", "proceed"

        @request_property
        def model(self):
            model = schema.Schema("SetQuantitiesForm", members = [
                schema.Collection("quantity",
                    items = schema.Integer(min = 1),
                    length = len(Basket.get().purchases)
                )
            ])

            @extend(model["quantity"].items)
            def translate_error(member, error, language = None, **kwargs):
                if isinstance(error, schema.exceptions.MinValueError):
                    return translations(
                        "SetQuantitiesForm-MinValueError",
                        language,
                        **kwargs
                    )
                else:
                    return call_base(error, language, **kwargs)

            return model

        def submit(self):
            Form.submit(self)

            for purchase, quantity in zip(
                Basket.get().purchases,
                self.instance["quantity"]
            ):
                purchase.quantity = quantity

            Basket.store()
            notify_user(
                translations("woost.extensions.ecommerce."
                             "set_quantities_notice"),
                category = "success"
            )

            if self.controller.action != "proceed":
                Location.get_current().go("GET")

    class NextStepForm(ProceedForm):
        process_after = "set_quantities_form",

    class PreviousStepForm(BackForm):
        pass

    class DeletePurchaseForm(Form):
        """A form that removes a product from the shopping basket."""

        actions = "delete_purchase",

        @request_property
        def model(self):
            return schema.Schema("DeletePurchaseForm", members = [
                schema.Reference("purchase",
                    type = ECommercePurchase,
                    required = True,
                    enumeration = lambda ctx: Basket.get().purchases
                )
            ])

        def submit(self):
            Form.submit(self)
            purchase = self.instance["purchase"]
            product = purchase.product
            purchase.delete()
            Basket.store()
            notify_user(
                translations(
                    "woost.extensions.ecommerce.delete_purchase_notice",
                    product = product
                ),
                category = "success"
            )
            Location.get_current().go("GET")

    class EmptyBasketForm(Form):
        """A form that removes all products from the shopping basket."""

        actions = "empty_basket",

        def submit(self):
            Basket.empty()
            notify_user(
                translations("woost.extensions.ecommerce."
                             "empty_basket_notice"),
                category = "success"
            )
            Location.get_current().go("GET")




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import re
import sys
from time import time
from simplejson import loads
from urllib import urlopen
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from woost.models import Extension, Language

translations.define("CountriesExtension",
    ca = u"Països",
    es = u"Países",
    en = u"Countries"
)

translations.define("CountriesExtension-plural",
    ca = u"Països",
    es = u"Países",
    en = u"Countries"
)

translations.define("CountriesExtension.update_frequency",
    ca = u"Freqüència d'actualització",
    es = u"Frecuencia de actualización",
    en = u"Update frequency"
)

translations.define("CountriesExtension.update_frequency-explanation",
    ca = u"El nombre de dies entre actualitzacions de la llista de paisos",
    es = u"El número de días entre actualizaciones de la lista de países",
    en = u"Number of days between updates to the country list"
)

SECONDS_IN_A_DAY = 60 * 60 * 24


class CountriesExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona accés a la llista de països del món,
            actualitzada a través d'Internet.""",
            "ca"
        )
        self.set("description",
            u"""Proporciona acceso a la lista de países del mundo, actualizada
            a través de Internet.""",
            "es"
        )
        self.set("description",
            u"""Provides a list of world countries, automatically updated from
            the Internet.""",
            "en"
        )

    last_update = None

    update_frequency = schema.Integer(
        min = 1,
        required = True,
        default = 15
    )

    def _load(self):

        from woost.extensions.countries import country, strings

        now = time()

        if self.last_update is None \
        or now - self.last_update >= self.update_frequency * SECONDS_IN_A_DAY:
            try:
                self.update_country_list()
            except:
                pass
            else:
                self.last_update = now
                datastore.commit()

    def update_country_list(self):
        from woost.extensions.countries.country import Country
        error = None

        database_modified = False
        service_uri = "http://www.lonelydrops.com/drops/1.0/list/%s/countries"

        data_expr = re.compile(
            r"var\s+drops_countries_[a-z]{2}\s*=\s*([^;]+)",
            re.DOTALL
        )
        json_property_normalization = re.compile(
            r"([a-zA-Z_$][a-zA-Z_$0-9]*)(?=\s*:)"
        )

        for language in Language.codes:
            try:
                javascript = urlopen(service_uri % language).read()
            except Exception, error:
                pass
            else:
                match = data_expr.search(javascript)

                if not match:
                    sys.stderr.write(
                        "The country list provider returned data in an "
                        "unexpected format (requested translation: %s)" % language
                    )
                    continue

                json = match.group(1)
                json = json_property_normalization.sub(r'"\1"', json)

                try:
                    data = loads(json)
                except Exception, error:
                    sys.stderr.write(
                        "The country list provider returned invalid data: %s"
                        "(requested translation: %s)\n" % (error, language)
                    )
                    continue

                for country_data in data:

                    # Ignore continents
                    if country_data.get("isParent"):
                        continue

                    name = country_data["label"]
                    iso_code = country_data["value"].lower()
                    country = Country.get_instance(iso_code = iso_code)

                    # New country
                    if country is None:
                        country = Country()
                        country.iso_code = iso_code
                        country.set("country_name", name, language)
                        country.insert()
                        database_modified = True

                    # Modified country
                    elif country.get("country_name", language) != name:
                        country.set("country_name", name, language)
                        database_modified = True

        if error:
            raise error

        return database_modified




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""

from cocktail import schema
from cocktail.translations import translations
from woost.models import (
    Extension,
    extension_translations,
    Site,
    StandardPage,
    Controller,
    Template,
    get_current_user,
    CreatePermission,
    ModifyPermission,
    DeletePermission,
    PermissionExpression
)

translations.define("CampaignMonitorExtension",
    ca = u"Campaign Monitor",
    es = u"Campaign Monitor",
    en = u"Campaign Monitor"
)

translations.define("CampaignMonitorExtension-plural",
    ca = u"Campaign Monitor",
    es = u"Campaign Monitor",
    en = u"Campaign Monitor"
)

class CampaignMonitorExtension(Extension):

    initialized = False

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet la integració amb el sistema de mailing Campaign Monitor""",
            "ca"
        )
        self.set("description",
            u"""Permite la integración con el sistema de mailing Campaign Monitor""",
            "es"
        )
        self.set("description",
            u"""Allows the integration with the Campaign Monitor mailing system""",
            "en"
        )

    def _load(self):

        from woost.extensions.campaignmonitor import (
            campaignmonitorlist,
            strings,
            useraction
        )

        from woost.extensions.campaignmonitor.campaignmonitorsubscriptionpage \
            import CampaignMonitorSubscriptionPage

        # Setup the synchronization view
        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController
        from woost.extensions.campaignmonitor.synccontroller import \
            SyncCampaignMonitorListsController

        BackOfficeController.sync_campaign_monitor_lists = \
            SyncCampaignMonitorListsController

        # Extension fields
        from woost.extensions.campaignmonitor.campaignmonitorlist \
            import CampaignMonitorList

        CampaignMonitorExtension.add_member(
            schema.String(
                "api_key",
                required = True,
                text_search = False
            )
        )

        CampaignMonitorExtension.add_member(
            schema.String(
                "client_id",
                required = True,
                text_search = False
            )
        )

        CampaignMonitorExtension.add_member(
            schema.Collection(
                "lists",
                items = schema.Reference(
                    type = CampaignMonitorList
                ),
                integral = True,
                related_end = schema.Reference()
            )
        )

        self.install()

    def _install(self):

        # Subscription controller
        campaign_monitor_controller = self._create_asset(
            Controller,
            "subscription_controller",
            python_name =
                "woost.extensions.campaignmonitor.campaignmonitorcontroller."
                "CampaignMonitorController",
            title = extension_translations
        )

        # Unsubscription controller
        unsubscription_controller = self._create_asset(
            Controller,
            "unsubscription_controller",
            python_name =
                "woost.extensions.campaignmonitor."
                "campaignmonitorunsubscriptioncontroller."
                "CampaignMonitorUnsubscriptionController",
            title = extension_translations
        )

        # Subscription template
        subscription_view = self._create_asset(
            Template,
            "subscription_template",
            identifier =
                "woost.extensions.campaignmonitor."
                "CampaignMonitorSubscriptionView",
            engine = "cocktail",
            title = extension_translations
        )

        # Unsubscription template
        subscription_view = self._create_asset(
            Template,
            "unsubscription_template",
            identifier =
                "woost.extensions.campaignmonitor."
                "CampaignMonitorUnsubscriptionView",
            engine = "cocktail",
            title = extension_translations
        )

        # Default subscription page
        subscription_page = self._create_asset(
            CampaignMonitorSubscriptionPage,
            "subscription_page",
            title = extension_translations,
            parent = Site.main.home,
            hidden = True
        )

        # Standard Template
        standard_template = Template.get_instance(
            qname = "woost.standard_template"
        )

        # Default pending page
        pending_page = self._create_asset(
            StandardPage,
            "pending_page",
            title = extension_translations,
            body = extension_translations,
            parent = Site.main.home,
            template = standard_template,
            hidden = True
        )

        # Default confirmation success page
        confirmation_success_page = self._create_asset(
            StandardPage,
            "confirmation_success_page",
            title = extension_translations,
            body = extension_translations,
            parent = Site.main.home,
            template = standard_template,
            hidden = True
        )

        # Default unsubscribe page
        unsubscribe_page = self._create_asset(
            StandardPage,
            "unsubscribe_page",
            title = extension_translations,
            body = extension_translations,
            controller = unsubscription_controller,
            template = unsubscription_view,
            parent = Site.main.home,
            hidden = True
        )

    def synchronize_lists(self, restricted = False):
        """Synchronizes the list of Lists of the given Campaign Monitor account
        with the site's database.

        The method queries Campaing Monitor API, retrieving the list of
        Lists for the indicated account and comparing it with the set of
        already known Lists (from previous executions of this method). The
        local database will be updated as follows:

            * Lists declared by Campaing Monitor that are not present in the
              database will generate new instances.
            * Lists that exist on both ends will be updated with the data
              provided by the Campaing Monitor service (only non editable
              members will be updated, so that data entered by users in the
              backoffice is preserved).
            * Lists that were instantiated in a previous run but which have
              been deleted at the Campaign Monitor side will be removed from
              the database.

        @param restricted: Indicates if access control should be applied to the
            operations performed by the method.
        @type restricted: bool
        """
        from campaign_monitor_api import CampaignMonitorApi
        from cocktail.controllers import context
        from cocktail.controllers.location import Location
        from woost.extensions.campaignmonitor.campaignmonitorlist import \
            CampaignMonitorList

        if restricted:
            user = get_current_user()

        api = CampaignMonitorApi(
            self.api_key,
            self.client_id
        )

        remote_lists = set()

        for list_data in api.client_get_lists():

            list_id = list_data["ListID"]
            cmlist = CampaignMonitorList.get_instance(list_id = list_id)
            remote_lists.add(list_id)

            # Check permissions
            if restricted and not user.has_permission(
                CreatePermission if cmlist is None else ModifyPermission,
                target = (cmlist or CampaignMonitorList)
            ):
                continue

            # Create new lists
            if cmlist is None:
                cmlist = CampaignMonitorList()
                cmlist.insert()
                cmlist.list_id = list_id
                self.lists.append(cmlist)

            # Modify new or updated lists with remote data
            cmlist.title = list_data.get("Name")

            list_detail_data = api.list_get_detail(list_id)

            # Modify remote lists with page urls

            def absolute_uri(publishable, *args, **kwargs):
                location = Location.get_current_host()
                location.path_info = context["cms"].uri(
                    publishable,
                    *args,
                    **kwargs
                )
                return str(location)

            if cmlist.unsubscribe_page:
                unsubscribe_page = "%s?user=[email]" % absolute_uri(cmlist.unsubscribe_page)
            else:
                unsubscribe_page = list_detail_data.get("UnsubscribePage")

            if cmlist.confirmation_success_page:
                confirmation_success_page = absolute_uri(cmlist.confirmation_success_page)
            else:
                confirmation_success_page = list_detail_data.get("ConfirmationSuccessPage")

            api.list_update(
                list_id,
                list_detail_data.get("Title"),
                unsubscribe_page,
                list_detail_data.get("ConfirmOptIn"),
                confirmation_success_page
            )

        # Delete lists that have been deleted from the user account
        missing_lists = CampaignMonitorList.select(
            CampaignMonitorList.list_id.not_one_of(remote_lists),
        )

        if restricted:
            missing_lists.add_filter(
                PermissionExpression(user, DeletePermission)
            )

        missing_lists.delete_items()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from cocktail.translations import translations
from cocktail.html import Element
from cocktail.html.databoundcontrol import data_bound, bind_member
from cocktail.html.datadisplay import (
    NO_SELECTION,
    SINGLE_SELECTION,
    MULTIPLE_SELECTION
)
from woost.views.contenttypetree import ContentTypeTree


class ContentTypePicker(ContentTypeTree):

    name = None
    value = None

    selection_mode = SINGLE_SELECTION

    empty_option_displayed = True

    def _build(self):
        data_bound(self)
        ContentTypeTree._build(self)
        self.add_resource("/resources/scripts/ContentTypePicker.js")

    def _ready(self):

        self.set_client_param("selectionMode", self.selection_mode)
        self.add_client_translation("woost.views.ContentTypePicker select")
        self.add_client_translation("woost.views.ContentTypePicker accept")
        self.add_client_translation("woost.views.ContentTypePicker cancel")

        self.set_client_variable(
            "cocktail.NO_SELECTION", NO_SELECTION)
        self.set_client_variable(
            "cocktail.SINGLE_SELECTION", SINGLE_SELECTION)
        self.set_client_variable(
            "cocktail.MULTIPLE_SELECTION", MULTIPLE_SELECTION)

        if self.selection_mode == SINGLE_SELECTION:
            self._control_type = "radio"
        elif self.selection_mode == MULTIPLE_SELECTION:
            self._control_type = "checkbox"
            self.empty_option_displayed = False

        if self.member and self.name is None and self.member.name:
            self.name = self.member.name

        if self.empty_option_displayed:
            self.append(self.create_empty_option())

        ContentTypeTree._ready(self)

    def create_empty_option(self):

        entry = Element("li")

        entry.control = Element("input",
            type = self._control_type,
            name = self.name,
            value = "",
            checked = self.value is None
        )
        bind_member(self, entry.control)
        entry.append(entry.control)

        entry.label = Element("label")
        entry.label.add_class("entry_label")
        entry.label.add_class("empty_option")
        entry.label.append(translations("woost.views.ContentTypePicker.empty_label"))
        entry.append(entry.label)

        entry.label["for"] = entry.control.require_id()

        return entry

    def create_entry(self, content_type):

        entry = ContentTypeTree.create_entry(self, content_type)

        entry.control = Element("input",
            type = self._control_type,
            name = self.name,
            value = content_type.full_name,
            checked = self.value and (
                content_type is self.value
                if self.selection_mode == SINGLE_SELECTION
                else content_type in self.value
            )
        )

        bind_member(self, entry.control)
        entry.insert(0, entry.control)
        entry.label["for"] = entry.control.require_id()
        return entry

    def create_label(self, content_type):
        label = ContentTypeTree.create_label(self, content_type)
        label.tag = "label"
        return label




#-*- coding: utf-8 -*-
u"""
Site specific translation strings.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
import _PROJECT_MODULE_.translations.strings




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from time import time
from simplejson import loads
from urllib import urlopen
from cocktail.iteration import first
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import transaction
from woost.models import Extension

translations.define("LocationsExtension",
    ca = u"Localitats",
    es = u"Localidad",
    en = u"Locations"
)

translations.define("LocationsExtension.service_uri",
    ca = u"URL del repositori de dades",
    es = u"URL del repositorio de datos",
    en = u"URL of the data repository"
)

translations.define("LocationsExtension.update_frequency",
    ca = u"Freqüència d'actualització",
    es = u"Frecuencia de actualización",
    en = u"Update frequency"
)

translations.define("LocationsExtension.update_frequency-explanation",
    ca = u"El nombre de dies entre actualitzacions de la llista de localitats."
         u" Deixar en blanc per desactivar les actualitzacions.",
    es = u"El número de días entre actualizaciones de la lista de localidades."
         u" Dejar en blanco para desactivar las actualizaciones.",
    en = u"Number of days between updates to the list of locations. "
         u"Leave blank to disable updates."
)

translations.define("LocationsExtension.updated_location_types",
    ca = u"Tipus de localització a obtenir",
    es = u"Tipos de localización a obtener",
    en = u"Location types included in updates"
)

translations.define("LocationsExtension.updated_subset",
    ca = u"Subconjunt de localitats a actualitzar",
    es = u"Subconjunto de localidades a actualizar",
    en = u"Subset of locations included in updates"
)

translations.define("LocationsExtension.updated_subset-explanation",
    ca = u"Llistat de codis qualificats de localitat (per exemple, EU-ES-CT, "
         u"EU-ES-CV per seleccionar Catalunya i la Comunitat Valenciana).",
    es = u"Listado de códigos cualificados de localidad (por ejemplo, "
         u"EU-ES-CT, EU-ES-CV para seleccionar Cataluña y la Comunidad "
         u"Valenciana).",
    en = u"List of qualified location codes (ie. EU-ES-CT, EU-ES-CV to "
         u"select Catalonia and the Valencian Country)."
)

SECONDS_IN_A_DAY = 60 * 60 * 24


class LocationsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Proporciona accés a la llista de països del món i les seves
            regions, actualitzada a través d'Internet.""",
            "ca"
        )
        self.set("description",
            u"""Proporciona acceso a la lista de países del mundo y sus
            regiones, actualizada a través de Internet.""",
            "es"
        )
        self.set("description",
            u"""Provides a list of world countries and their regions,
            automatically updated from the Internet.""",
            "en"
        )

    last_update = None

    members_order = [
        "service_uri",
        "update_frequency",
        "updated_location_types",
        "updated_subset"
    ]

    service_uri = schema.String(
        required = True,
        default = "http://services.woost.info/locations",
        text_search = False
    )

    update_frequency = schema.Integer(
        min = 1,
        default = 15
    )

    updated_location_types = schema.Collection(
        edit_inline = True,
        default = [
            "continent",
            "country",
            "autonomous_community",
            "province",
            "town"
        ],
        items = schema.String(
            required = True,
            enumeration = [
                "continent",
                "country",
                "autonomous_community",
                "province",
                "town"
            ],
            translate_value = lambda value, language = None, **kwargs:
                "" if not value
                else translations(
                    "woost.extensions.locations.location_types." + value,
                    language,
                    **kwargs
                )
        ),
        text_search = False
    )

    updated_subset = schema.Collection(
        items = schema.String(
            required = True
        ),
        edit_control = "cocktail.html.TextArea"
    )

    def _load(self):
        from woost.extensions.locations import location, strings

        if self.should_update():
            transaction(
                self.sync_locations,
                desist = lambda: not self.should_update()
            )

    def should_update(self):

        if self.last_update is None:
            return True

        if self.update_frequency is None:
            return False

        return time() - self.last_update >= self.update_frequency * SECONDS_IN_A_DAY

    def sync_locations(self):
        """Populate the locations database with the data provided by the web
        service.
        """
        text_data = urlopen(self.service_uri).read()
        json_data = loads(text_data)

        for record in json_data:
            self._process_record(record)

        self.last_update = time()

    def _process_record(self, record, parent = None, context = None):

        from woost.extensions.locations.location import Location

        context = context.copy() if context else {}
        code = record["code"]
        location = None

        if "full_code" in context:
            context["full_code"] += "-" + code
        else:
            context["full_code"] = code

        if self._should_add_location(record, parent, context):

            # Update an existing location
            if parent:
                location = first(
                    child
                    for child in parent.locations
                    if child.code == code
                )
            else:
                location = first(Location.select([
                    Location.parent.equal(None),
                    Location.code.equal(code)
                ]))

            # Create a new location
            if location is None:
                location = Location()
                location.parent = parent
                location.code = code
                location.insert()

            location.location_type = record["type"]

            for lang, value in record["name"].iteritems():
                if isinstance(value, str):
                    value = value.decode("utf-8")
                location.set("location_name", value, lang)

        # Process child records
        children = record.get("locations")
        if children:
            for child_record in children:
                self._process_record(child_record, location, context)

    def _should_add_location(self, record, parent, context):

        # Filter by tree subset
        if self.updated_subset:
            if not context.get("inside_subset") \
            and context["full_code"] not in self.updated_subset:
                return False
            else:
                context["inside_subset"] = True

        # Filter by location type
        if record["type"] not in self.updated_location_types:
            return False

        return True




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from hashlib import sha1
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail import schema
from woost.models.site import Site
from woost.models.item import Item
from woost.models.role import Role
from woost.models.messagestyles import (
    role_style,
    permission_style,
    permission_check_style,
    permission_param_style,
    authorized_style,
    unauthorized_style
)

verbose = False

CMS_TRANSLATIONS = "en", "es", "ca"


class User(Item):
    """An application user.

    @ivar anonymous: Indicates if this is the user that represents
        anonymous users.

        To handle anonymous access to the application, an instance of the
        User class is designated to represent anonymous users. This allows
        uniform treatment of both anonymous and authenticated access (ie.
        authorization checks).

        This property indicates if the user it is invoked on is the
        instance used to represent anonymous users.

    @type anonymous: bool

    @ivar encryption_method: The hashing algorithm used to encrypt user
        passwords. Should be a reference to one of the algorithms provided
        by the L{hashlib} module.
    """
    edit_form = "woost.views.UserForm"
    edit_node_class = \
        "woost.controllers.backoffice.usereditnode.UserEditNode"

    encryption_method = sha1

    anonymous = False

    members_order = [
        "enabled",
        "email",
        "password",
        "prefered_language"
    ]

    email = schema.String(
        required = True,
        unique = True,
        descriptive = True,
        max = 255,
        indexed = True,
        format = "^.+@.+$"
    )

    password = schema.String(
        required = True,
        listable = False,
        listed_by_default = False,
        searchable = False,
        text_search = False,
        min = 8,
        visible_in_detail_view = False,
        edit_control = "cocktail.html.PasswordBox"
    )

    prefered_language = schema.String(
        required = True,
        default = schema.DynamicDefault(
            lambda: Site.main.backoffice_language
        ),
        enumeration = lambda ctx: Site.backoffice_language.enumeration,
        translate_value = lambda value, language = None, **kwargs:
            "" if value is None else translations(value, language, **kwargs),
        text_search = False
    )

    roles = schema.Collection(
        items = "woost.models.Role",
        bidirectional = True
    )

    enabled = schema.Boolean(
        required = True,
        default = True
    )

    def encryption(self, data):

        if self.encryption_method:

            if isinstance(data, unicode):
                data = data.encode("utf-8")

            data = self.encryption_method(data).digest()

        return data

    @event_handler
    def handle_changing(cls, event):

        if event.member is cls.password \
        and event.value is not None:
            event.value = event.source.encryption(event.value)

    def test_password(self, password):
        """Indicates if the user's password matches the given string.

        @param password: An unencrypted string to tests against the user's
            encrypted password.
        @type password: str

        @return: True if the passwords match, False otherwise.
        @rtype: bool
        """
        if password:
            return self.encryption(password) == self.password
        else:
            return not self.password

    def iter_roles(self, recursive = True):
        """Obtains all the roles that apply to the user.

        The following roles can be yielded:

            * The user's L{explicit roles<roles>} will be yielded if defined
            * An 'authenticated' role will be yielded if the user is not
              L{anonymous}
            * An 'everybody' role that applies to all users will always be
              yielded

        Roles are sorted in descending relevancy order.

        @return: An iterable sequence of roles that apply to the user.
        @rtype: L{Role}
        """
        explicit_roles = self.roles
        if explicit_roles:
            if recursive:
                for role in explicit_roles:
                    for ancestor_role in role.iter_roles():
                        yield ancestor_role
            else:
                for role in explicit_roles:
                    yield role

        for role in self.iter_implicit_roles():
            yield role

    def iter_implicit_roles(self):

        if not self.anonymous:
            yield Role.require_instance(qname = "woost.authenticated")

        yield Role.require_instance(qname = "woost.everybody")

    def has_role(self, role):
        """Determines if the user has been granted the indicated role.

        This takes into account inherited and implicit roles.

        @return: True if the user possesses the given role, False otherwise.
        @rtype: bool
        """
        for user_role in self.iter_roles():
            if user_role is role:
                return True

        return False

    def iter_permissions(self, permission_type = None):
        """Iterates over the permissions granted to the user's roles.

        This method yields all permissions that are granted to any of the
        user's roles.

        @param permission_type: If given, restricts the list of returned
            permissions to those of the given type (or a subclass of that
            type). By default, all permissions are yielded, regardless of their
            type.
        @type permission_type: L{Permission} subclass

        @return: An iterable sequence of permissions granted to any of the
            user's roles.
        @rtype: L{Permission} sequence
        """
        for role in self.iter_roles(False):
            for permission in role.iter_permissions(permission_type):
                yield permission

    def has_permission(self,
        permission_type,
        verbose = None,
        **context):
        """Determines if the user is given permission to perform an action.

        @param permission_type: The kind of permission to assert.
        @type permission_type: L{Permission} subclass

        @param verbose: A boolean flag that enables or disables on screen
            reporting of the checks executed by this method. Can help debugging
            authorization issues. When set, overrides the value of the module
            L{verbose} flag.
        @type verbose: bool

        @param context: Keyword parameters to supply to each tested permission.
            Each parameter will be forwarded to the permission's
            L{match<woost.models.permission.Permission.match>} method. Each
            subclass or L{Permission<woost.models.permission.Permission>}
            can define its set of required parameters, and all of its
            subclasses must implement exactly that set.

        @return: True if the user is granted permission, else False.
        @rtype: bool
        """
        if verbose is None:
            verbose = globals()["verbose"]

        if verbose:
            role = None
            print permission_check_style(translations(permission_type.name))
            print permission_param_style("user", translations(self))

            for key, value in context.iteritems():
                if isinstance(value, type):
                    value = translations(value.name)
                else:
                    trans = translations(value)
                    if trans and isinstance(value, Item):
                        value = u"%s (%s)" % (unicode(value), trans)
                    else:
                        value = trans or unicode(value)

                print permission_param_style(key, value)

        permissions = self.iter_permissions(permission_type)

        for permission in permissions:

            if verbose:
                new_role = permission.role
                if new_role is not role:
                    role = new_role
                    print role_style(translations(role))

                print permission_style(
                    "#%d. %s" % (permission.id, translations(permission))
                )

            if permission.match(verbose = verbose, **context):

                if permission.authorized:
                    if verbose:
                        print authorized_style("authorized")
                    return True
                else:
                    break

        return permission_type.permission_not_found(
            self,
            verbose = verbose,
            **context
        )

    def require_permission(self, permission_type, verbose = None, **context):
        """Asserts the user has permission to perform an action.

        This method is similar to L{has_permission}, but instead of returning a
        boolean value, it will raise an exception if the permission is not
        granted.

        @param permission_type: The kind of permission to assert.
        @type permission_type: L{Permission} subclass

        @param verbose: A boolean flag that enables or disables on screen
            reporting of the checks executed by this method. Can help debugging
            authorization issues. When set, overrides the value of the module
            L{verbose} flag.
        @type verbose: bool

        @param context: Keyword parameters to supply to each tested permission.
            Each parameter will be forwarded to the permission's
            L{match<woost.models.permission.Permission.match>} method. Each
            subclass or L{Permission<woost.models.permission.Permission>}
            can define its set of required parameters, and all of its
            subclasses must implement exactly that set.

        @raise L{AuthorizationError}: Raised if the user doesn't have the
            specified permission.
        """
        if not self.has_permission(
            permission_type,
            verbose = verbose,
            **context
        ):
            raise AuthorizationError(
                user = self,
                permission_type = permission_type,
                context = context
            )


class AuthorizationError(Exception):
    """An exception raised when a user attempts an unauthorized action.

    @ivar user: The user that attempted the action.
    @type user: L{User}

    @ivar permission_type: The kind of permission lacked by the user.
    @type permission_type:
        L{Permission<woost.models.permission.Permission>} subclass

    @ivar context: A mapping with additional parameters providing context to
        the attempted action.
    @type context: dict
    """
    user = None
    permission_type = None
    context = None

    def __init__(self, user = None, permission_type = None, context = None):
        self.user = user
        self.permission_type = permission_type
        self.context = context

